<?php

if ( !defined( 'ABSPATH' ) ) exit();


if ( ! class_exists('OVAM_Admin_Ajax') ) {
	
	class OVAM_Admin_Ajax {

		public function __construct(){
			$hooks = array(
				'ova_megamenu_add_row',
				'ova_megamenu_add_column',
				'ova_megamenu_add_widget',
				'ova_megamenu_add_page_item',
				'ova_megamenu_filter_page',
				'ova_megamenu_add_post_item',
				'ova_megamenu_filter_post',
				'ova_megamenu_add_custom_link',
				'ova_megamenu_add_category_item',
				'ova_megamenu_filter_categories',
				'ova_megamenu_add_custom_tax_item',
				'ova_megamenu_filter_custom_taxonomies',
				'ova_megamenu_filter_custom_post_type',
				'ova_megamenu_add_custom_post_item',
				'ova_megamenu_select_font',
				'ova_megamenu_export_settings',
				'ova_megamenu_import_popup',
				'ova_megamenu_import_settings',
				'ova_megamenu_reset_settings',
				'ova_megamenu_copy_update',
			);

			foreach ($hooks as $hook) {
				add_action( 'wp_ajax_'.$hook, array( $this, $hook ) );
			}
			
		}

		public function ova_megamenu_add_row(){
			$response = '';
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_megamenu_action' ) ) {
				echo $response;
				wp_die();
			}
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : 0;
			$column = 0;
			$widget = 0;
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			$widgets = ova_megamenu_get_type_item();
			$column_width = 100;
			ob_start();
			?>
			<li class="ova_megamenu_row card" data-row="<?php echo esc_attr( $row ); ?>">
				<ul class="ova_megamenu_columns">
					<li class="ova_megamenu_column card" style="width: 100%;"
						data-column="<?php echo esc_attr( $column ); ?>"
						data-row="<?php echo esc_attr( $row ); ?>">
						<button type="button" class="ova_megamenu_remove_column button button-small button-secondary">&#x2715;</button>
						<button type="button" class="ova_megamenu_edit_width button button-small button-secondary"><span class="dashicons dashicons-edit"></span></button>
						<div class="ova_megamenu_resize"></div>
						<div class="show_column_percent"></div>
						<input type="number"
							class="ova_megamenu_column_width"
							min="1" step="0.1"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][column_width]" value="<?php echo esc_attr( $column_width ); ?>">
						<ul class="ova_megamenu_widgets">
							<li class="ova_megamenu_widget_item card"
							data-widget="<?php echo esc_attr( $widget ); ?>"
							data-column="<?php echo esc_attr( $column ); ?>"
							data-row="<?php echo esc_attr( $row ); ?>">
									<button type="button" class="ova_megamenu_remove_widget button button-small button-primary">&#x2715;</button>
									<table class="form-table">
										<tbody>

											<tr>
												<td>
													<label for=""><?php esc_html_e( 'Choose a type', 'ova-megamenu' ); ?></label>
													<select class="ova_megamenu_widget" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][type]" id="">
														<option selected="selected" value=""><?php esc_html_e( '--- Type ---', 'ova-megamenu' ); ?></option>
														<?php if ( ! empty( $widgets ) ): ?>
															<?php foreach ($widgets as $key => $value): ?>
																<option value="<?php echo esc_attr( $key ); ?>"><?php echo esc_html( $value ); ?></option>
															<?php endforeach; ?>
														<?php endif; ?>
													</select>
												</td>
											</tr>

											<tr>
												
												<td>
													<label for=""><?php esc_html_e( 'Heading', 'ova-megamenu' ); ?>
													<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][heading]" type="text" placeholder="<?php esc_attr_e( 'Enter heading', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
												</td>
											</tr>

											<tr class="ova_megamenu_field" data-id="shortcode">
												
												<td>
													<label for=""><?php esc_html_e( 'Shortcode', 'ova-megamenu' ); ?>
													<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][shortcode]" type="text" placeholder="<?php esc_attr_e( 'Enter shortcode', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
												</td>
											</tr>

											<tr class="ova_megamenu_field" data-id="text">
												
												<td>
													<label for=""><?php esc_html_e( 'Text', 'ova-megamenu' ); ?>
													<?php
														$settings  = array(
															'textarea_name' => 'ova_megamenu_'.$this_tab.'['.$row.']['.$column.'][widget]['.$widget.'][text]',
															'textarea_rows' => 10,
															'editor_height' => 230,
															'wpautop' => false,
															'quicktags' => true,
														);
														$id = 'ova_megamenu_'.uniqid();
														wp_editor( wpautop(''), $id, $settings );
													?>
													<script type="text/javascript">
														var id = "<?php echo $id; ?>";
														tinymce.execCommand( 'mceAddEditor', true, id );
														quicktags({id: id});
													</script>
												</td>
											</tr>

											<tr class="ova_megamenu_field" data-id="menu">
												<td>
													<label for=""><?php esc_html_e( 'Type:', 'ova-megamenu' ); ?></label>
													
													<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" checked value="add-item">
														<?php esc_html_e( 'Add items', 'ova-megamenu' ); ?>
													</span>
													<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" value="select-menu">
														<?php esc_html_e( 'Select menu', 'ova-megamenu' ); ?>
													</span>
												</td>

											</tr>

											<tr class="ova_megamenu_sub_field" data-id="select-menu">
												<?php $nav_menus = ova_megamenu_get_menus(); ?>
												<td>
													<label for=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></label>
													<select name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu]" id="">
														<option selected="selected" value=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></option>
														<?php if ( ! empty( $nav_menus ) ): ?>
															<?php foreach ($nav_menus as $key => $value): ?>
																<option value="<?php echo esc_attr( $value->term_id ); ?>"><?php echo esc_html( $value->name ); ?></option>
															<?php endforeach; ?>
														<?php endif; ?>
													</select>
												</td>
											</tr>

											<tr class="ova_megamenu_sub_field" data-id="add-item">
																								
												<td>
													<div class="ova_header">

														<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
															<input type="checkbox" class="menu_item_bulk_select" value="1">
															<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
														</button>

													</div>
													<ul class="list-menu-item"></ul>
													<div class="ova_footer">
																							
														<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
															<input type="checkbox" class="menu_item_bulk_select" value="1">
															<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
														</button>
													
														<a href="#" class="ova_menu_item_remove"><?php esc_html_e( 'Remove Selected Items', 'ova-megamenu' ); ?></a>
														
													</div>
													<button type="button" data-btn="<?php esc_attr_e( 'Close', 'ova-megamenu' ); ?>" data-title="<?php esc_attr_e( 'Add menu item', 'ova-megamenu' ); ?>" class="ova_megamenu_add_menu_item button button-secondary"><?php esc_html_e( 'Add menu item', 'ova-megamenu' ); ?></button>
												</td>
											</tr>

										</tbody>
									</table>
							</li>
						</ul>

							<button type="button" class="ova_megamenu_add_widget button button-secondary"><?php esc_html_e( 'Add widget', 'ova-megamenu' ); ?></button>

					</li>
				</ul>
				
				<button type="button" class="ova_megamenu_remove_row button button-small button-primary">&#x2715;</button>

					<button type="button" class="ova_megamenu_add_column button button-secondary"><?php esc_html_e( 'Add column', 'ova-megamenu' ); ?></button>

				

			</li>
			<?php
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_column(){
			$response = '';
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_megamenu_action' ) ) {
				echo $response;
				wp_die();
			}
			ob_start();
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : 0;
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : 0;
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			$widgets = ova_megamenu_get_type_item();
			$widget = 0;
			?>
			<li class="ova_megamenu_column card" data-column="<?php echo esc_attr( $column ); ?>" data-row="<?php echo esc_attr( $row ); ?>">
				<button type="button" class="ova_megamenu_remove_column button button-small button-secondary">&#x2715;</button>
				<button type="button" class="ova_megamenu_edit_width button button-small button-secondary"><span class="dashicons dashicons-edit"></span></button>
				<div class="ova_megamenu_resize"></div>
				<div class="show_column_percent"></div>
				<input type="number" class="ova_megamenu_column_width" min="1" step="0.1" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][column_width]" value="">
				<ul class="ova_megamenu_widgets">
					<li class="ova_megamenu_widget_item card"
					data-widget="<?php echo esc_attr( $widget ); ?>"
					data-column="<?php echo esc_attr( $column ); ?>"
					data-row="<?php echo esc_attr( $row ); ?>">

							<button type="button" class="ova_megamenu_remove_widget button button-small button-primary">&#x2715;</button>
							<table class="form-table">
								<tbody>

									<tr>
										<td>
											<label for=""><?php esc_html_e( 'Choose a type', 'ova-megamenu' ); ?></label>
											<select class="ova_megamenu_widget" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][type]" id="">
												<option selected="selected" value=""><?php esc_html_e( '--- Type ---', 'ova-megamenu' ); ?></option>
												<?php if ( ! empty( $widgets ) ): ?>
													<?php foreach ($widgets as $key => $value): ?>
														<option value="<?php echo esc_attr( $key ); ?>"><?php echo esc_html( $value ); ?></option>
													<?php endforeach; ?>
												<?php endif; ?>
											</select>
										</td>
									</tr>

									<tr>
										
										<td>
											<label for=""><?php esc_html_e( 'Heading', 'ova-megamenu' ); ?>
											<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][heading]" type="text" placeholder="<?php esc_attr_e( 'Enter heading', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
										</td>
									</tr>

									<tr class="ova_megamenu_field" data-id="shortcode">
										
										<td>
											<label for=""><?php esc_html_e( 'Shortcode', 'ova-megamenu' ); ?>
											<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][shortcode]" type="text" placeholder="<?php esc_attr_e( 'Enter shortcode', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
										</td>
									</tr>

									<tr class="ova_megamenu_field" data-id="text">
										
										<td>
											<label for=""><?php esc_html_e( 'Text', 'ova-megamenu' ); ?>
											<?php
												$settings  = array(
													'textarea_name' => 'ova_megamenu_'.$this_tab.'['.$row.']['.$column.'][widget]['.$widget.'][text]',
													'textarea_rows' => 10,
													'editor_height' => 230,
													'wpautop' => false,
													'quicktags' => true,
												);
												$id = 'ova_megamenu_'.uniqid();
												wp_editor( wpautop(''), $id, $settings );
											?>
											<script type="text/javascript">
												var id = "<?php echo $id; ?>";
												tinymce.execCommand( 'mceAddEditor', true, id );
												quicktags({id: id});
											</script>
										</td>
									</tr>

									<tr class="ova_megamenu_field" data-id="menu">
										<td>
											<label for=""><?php esc_html_e( 'Type:', 'ova-megamenu' ); ?></label>
											
											<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" checked value="add-item">
												<?php esc_html_e( 'Add items', 'ova-megamenu' ); ?>
											</span>
											<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" value="select-menu">
												<?php esc_html_e( 'Select menu', 'ova-megamenu' ); ?>
											</span>
										</td>

									</tr>

									<tr class="ova_megamenu_sub_field" data-id="select-menu">
										<?php $nav_menus = ova_megamenu_get_menus(); ?>
										<td>
											<label for=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></label>
											<select name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu]" id="">
												<option selected="selected" value=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></option>
												<?php if ( ! empty( $nav_menus ) ): ?>
													<?php foreach ($nav_menus as $key => $value): ?>
														<option value="<?php echo esc_attr( $value->term_id ); ?>"><?php echo esc_html( $value->name ); ?></option>
													<?php endforeach; ?>
												<?php endif; ?>
											</select>
										</td>
									</tr>

									<tr class="ova_megamenu_sub_field" data-id="add-item">
																						
										<td>
											<div class="ova_header">

												<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
													<input type="checkbox" class="menu_item_bulk_select" value="1">
													<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
												</button>

											</div>
											<ul class="list-menu-item"></ul>
											<div class="ova_footer">
																							
												<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
													<input type="checkbox" class="menu_item_bulk_select" value="1">
													<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
												</button>
											
												<a href="#" class="ova_menu_item_remove"><?php esc_html_e( 'Remove Selected Items', 'ova-megamenu' ); ?></a>
												
											</div>
											<button type="button" data-btn="<?php esc_attr_e( 'Close', 'ova-megamenu' ); ?>" data-title="<?php esc_attr_e( 'Add menu item', 'ova-megamenu' ); ?>" class="ova_megamenu_add_menu_item button button-secondary"><?php esc_html_e( 'Add menu item', 'ova-megamenu' ); ?></button>
										</td>
									</tr>

								</tbody>
							</table>
					</li>
				</ul>

					<button type="button" class="ova_megamenu_add_widget button button-secondary"><?php esc_html_e( 'Add widget', 'ova-megamenu' ); ?></button>

			</li>
			<?php
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_widget(){
			$response = '';
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_megamenu_action' ) ) {
				echo $response;
				wp_die();
			}
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : 0;
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : 0;
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : 0;
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			$widgets = ova_megamenu_get_type_item();
			$nav_menus = ova_megamenu_get_menus();
			ob_start();
			?>
			<li class="ova_megamenu_widget_item card"
			data-widget="<?php echo esc_attr( $widget ); ?>"
			data-column="<?php echo esc_attr( $column ); ?>"
			data-row="<?php echo esc_attr( $row ); ?>">
					<button type="button" class="ova_megamenu_remove_widget button button-small button-primary">&#x2715;</button>
					<table class="form-table">
					<tbody>

						<tr>
							<td>
								<label for=""><?php esc_html_e( 'Choose a type', 'ova-megamenu' ); ?></label>
								<select class="ova_megamenu_widget" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][type]" id="">
									<option selected="selected" value=""><?php esc_html_e( '--- Type ---', 'ova-megamenu' ); ?></option>
									<?php if ( ! empty( $widgets ) ): ?>
										<?php foreach ($widgets as $key => $value): ?>
											<option value="<?php echo esc_attr( $key ); ?>"><?php echo esc_html( $value ); ?></option>
										<?php endforeach; ?>
									<?php endif; ?>
								</select>
							</td>
						</tr>

						<tr>
							
							<td>
								<label for=""><?php esc_html_e( 'Heading', 'ova-megamenu' ); ?>
								<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][heading]" type="text" placeholder="<?php esc_attr_e( 'Enter heading', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
							</td>
						</tr>

						<tr class="ova_megamenu_field" data-id="shortcode">
							
							<td>
								<label for=""><?php esc_html_e( 'Shortcode', 'ova-megamenu' ); ?>
								<input name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][shortcode]" type="text" placeholder="<?php esc_attr_e( 'Enter shortcode', 'ova-megamenu' ); ?>" id="" value="" class="large-text">
							</td>
						</tr>

						<tr class="ova_megamenu_field" data-id="text">
							
							<td>
								<label for=""><?php esc_html_e( 'Text', 'ova-megamenu' ); ?>
								<?php
									$settings  = array(
										'textarea_name' => 'ova_megamenu_'.$this_tab.'['.$row.']['.$column.'][widget]['.$widget.'][text]',
										'textarea_rows' => 10,
										'editor_height' => 230,
										'wpautop' => false,
										'quicktags' => true,
									);
									$id = 'ova_megamenu_'.uniqid();
									wp_editor( wpautop(''), $id, $settings );
								?>
								<script type="text/javascript">
									var id = "<?php echo $id; ?>";
									tinymce.execCommand( 'mceAddEditor', true, id );
									quicktags({id: id});
								</script>
							</td>
						</tr>

						<tr class="ova_megamenu_field" data-id="menu">
							<td>
								<label for=""><?php esc_html_e( 'Type:', 'ova-megamenu' ); ?></label>
								
								<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" checked value="add-item">
									<?php esc_html_e( 'Add items', 'ova-megamenu' ); ?>
								</span>
								<span><input class="menu_type" name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_type]" type="radio" value="select-menu">
									<?php esc_html_e( 'Select menu', 'ova-megamenu' ); ?>
								</span>
							</td>

						</tr>

						<tr class="ova_megamenu_sub_field" data-id="select-menu">
							<?php $nav_menus = ova_megamenu_get_menus(); ?>
							<td>
								<label for=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></label>
								<select name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu]" id="">
									<option selected="selected" value=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></option>
									<?php if ( ! empty( $nav_menus ) ): ?>
										<?php foreach ($nav_menus as $key => $value): ?>
											<option value="<?php echo esc_attr( $value->term_id ); ?>"><?php echo esc_html( $value->name ); ?></option>
										<?php endforeach; ?>
									<?php endif; ?>
								</select>
							</td>
						</tr>

						<tr class="ova_megamenu_sub_field" data-id="add-item">
																			
							<td>
								<div class="ova_header">

									<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
										<input type="checkbox" class="menu_item_bulk_select" value="1">
										<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
									</button>

								</div>
								<ul class="list-menu-item"></ul>
								<div class="ova_footer">
																							
									<button type="button" class="ova_megamenu_menu_item_bulk_select button button-small button-secondary">
										<input type="checkbox" class="menu_item_bulk_select" value="1">
										<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
									</button>
								
									<a href="#" class="ova_menu_item_remove"><?php esc_html_e( 'Remove Selected Items', 'ova-megamenu' ); ?></a>
									
								</div>
								<button type="button" data-btn="<?php esc_attr_e( 'Close', 'ova-megamenu' ); ?>" data-title="<?php esc_attr_e( 'Add menu item', 'ova-megamenu' ); ?>" class="ova_megamenu_add_menu_item button button-secondary"><?php esc_html_e( 'Add menu item', 'ova-megamenu' ); ?></button>
							</td>
						</tr>

					</tbody>
				</table>
			</li>
			<?php
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_page_item(){
			$response = '';
			$page_ids = isset( $_POST['page_ids'] ) ? $_POST['page_ids'] : array();
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			ob_start();
				if ( ! empty( $page_ids ) ):
					$k = (int) $menu_item;
					foreach ( $page_ids as $page_id ) :
						$title = get_the_title( $page_id );
						$url = get_permalink( $page_id );
						?>
						<li class="menu-item">
							<h4 class="menu-title">
								<span class="input-title">
									<input type="checkbox" class="ova_check_menu_item" value="1">
									<span class="title">
										<?php echo esc_html( $title ); ?>
									</span>
								</span>
								<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
							</h4>
							<div class="content">
								<ul class="list-menu-item-field">
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
										placeholder=""
										value="<?php echo esc_attr( $title ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
										placeholder=""
										value="<?php echo esc_attr( $url ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title_attr large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_classes large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for="">
										<input
										type="checkbox"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
										value="1" />
										<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
										</label>
									</li>
									<li class="menu-item-field footer">
										<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
										<span class="separator">|</span>
										<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
									</li>
								</ul>
							</div>
						</li>
				<?php $k += 1; endforeach; endif;
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_filter_page(){
			$response = '';
			$title = isset( $_POST['title'] ) ? sanitize_text_field( $_POST['title'] ) : '';
			

			ob_start();
			if ( ! empty( $title ) ) {
				$pages = ova_megamenu_get_page_by_title( $title );
				if ( $pages && $pages->have_posts() ) {
					while ( $pages->have_posts() ) {
						$pages->the_post();
						$id = get_the_ID();
						$title = get_the_title();
						?>
						<li class="page-item">
							<label for="<?php echo 'page-'.esc_attr( $id  ); ?>">
								<input class="page_item" type="checkbox" id="<?php echo 'page-'.esc_attr( $id  ); ?>" value="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html( $title ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php esc_html_e( 'Page not found.', 'ova-megamenu' ); ?></p>
					<?php
				}
				wp_reset_postdata();
			} else {
				$pages = ova_megamenu_get_page_default();
				if ( ! empty( $pages ) ) {
					foreach ($pages as $page): ?>
						<li class="page-item">
							<label for="<?php echo 'page-'.esc_attr( $page->ID  ); ?>">
								<input class="page_item" type="checkbox" id="<?php echo 'page-'.esc_attr( $page->ID  ); ?>" value="<?php echo esc_attr( $page->ID ); ?>">
								<?php echo esc_html( $page->post_title ); ?>
							</label>
						</li>
					<?php endforeach;
				} else {
					?>
					<p><?php esc_html_e( 'Page not found.', 'ova-megamenu' ); ?></p>
					<?php
				}
			}
			
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_post_item(){
			$response = '';
			$post_ids = isset( $_POST['post_ids'] ) ? $_POST['post_ids'] : array();
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			ob_start();
				if ( ! empty( $post_ids ) ):
					$k = (int) $menu_item;
					foreach ( $post_ids as $post_id ) :
						$title = get_the_title( $post_id );
						$url = get_permalink( $post_id );
						?>
						<li class="menu-item">
							<h4 class="menu-title">
								<span class="input-title">
									<input type="checkbox" class="ova_check_menu_item" value="1">
									<span class="title">
										<?php echo esc_html( $title ); ?>
									</span>
								</span>
								<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
							</h4>
							<div class="content">
								<ul class="list-menu-item-field">
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
										placeholder=""
										value="<?php echo esc_attr( $title ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
										placeholder=""
										value="<?php echo esc_attr( $url ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title_attr large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_classes large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for="">
										<input
										type="checkbox"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
										value="1" />
										<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
										</label>
									</li>
									<li class="menu-item-field footer">
										<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
										<span class="separator">|</span>
										<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
									</li>
								</ul>
							</div>
						</li>
				<?php $k += 1; endforeach; endif;
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_filter_post(){
			$response = '';
			$title = isset( $_POST['title'] ) ? sanitize_text_field( $_POST['title'] ) : '';
			

			ob_start();
			if ( ! empty( $title ) ) {
				$posts = ova_megamenu_get_post_by_title( $title );
				if ( $posts && $posts->have_posts() ) {
					while ( $posts->have_posts() ) {
						$posts->the_post();
						$id = get_the_ID();
						$title = get_the_title();
						?>
						<li class="item post-item">
							<label for="<?php echo 'post-'.esc_attr( $id  ); ?>">
								<input type="checkbox" id="<?php echo 'page-'.esc_attr( $id  ); ?>" value="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html( $title ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php esc_html_e( 'Post not found.', 'ova-megamenu' ); ?></p>
					<?php
				}
				wp_reset_postdata();
			} else {
				$posts = ova_megamenu_get_post_default();
				if ( ! empty( $posts ) ) {
					foreach ( $posts as $post ): ?>
						<li class="item post-item">
							<label for="<?php echo 'post-'.esc_attr( $post->ID  ); ?>">
								<input type="checkbox" id="<?php echo 'post-'.esc_attr( $post->ID  ); ?>" value="<?php echo esc_attr( $post->ID ); ?>">
								<?php echo esc_html( $post->post_title ); ?>
							</label>
						</li>
					<?php endforeach;
				} else {
					?>
					<p><?php esc_html_e( 'Post not found.', 'ova-megamenu' ); ?></p>
					<?php
				}
			}
			
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_custom_link(){
			$response = '';
			$url = isset( $_POST['url'] ) ? sanitize_url( $_POST['url'] ) : '';
			$link_text = isset( $_POST['link_text'] ) ? sanitize_text_field( $_POST['link_text'] ) : '';
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			$k = (int) $menu_item;
			ob_start();
			?>
			<li class="menu-item">
				<h4 class="menu-title">
					<span class="input-title">
						<input type="checkbox" class="ova_check_menu_item" value="1">
						<span class="title">
							<?php echo esc_html( $link_text ); ?>
						</span>
					</span>
					<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
				</h4>
				<div class="content">
					<ul class="list-menu-item-field">
						<li class="menu-item-field">
							<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
							<input
							type="text"
							class="menu_item_title large-text"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
							placeholder=""
							value="<?php echo esc_attr( $link_text ); ?>" />
						</li>
						<li class="menu-item-field">
							<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
							<input
							type="text"
							class="large-text"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
							placeholder=""
							value="<?php echo esc_attr( $url ); ?>" />
						</li>
						<li class="menu-item-field">
							<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
							<input
							type="text"
							class="menu_item_title_attr large-text"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
							placeholder=""
							value="" />
						</li>
						<li class="menu-item-field">
							<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
							<input
							type="text"
							class="menu_item_classes large-text"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
							placeholder=""
							value="" />
						</li>
						<li class="menu-item-field">
							<label for="">
							<input
							type="checkbox"
							name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
							value="1" />
							<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
							</label>
						</li>
						<li class="menu-item-field footer">
							<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
							<span class="separator">|</span>
							<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
						</li>
					</ul>
				</div>
			</li>
			<?php
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_category_item(){
			$response = '';
			$category_ids = isset( $_POST['category_ids'] ) ? $_POST['category_ids'] : array();
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			ob_start();
				if ( ! empty( $category_ids ) ):
					$k = (int) $menu_item;
					foreach ( $category_ids as $category_id ) :
						$category = get_term( $category_id );
						$title = $category->name;
						$url = get_term_link( $category );
						?>
						<li class="menu-item">
							<h4 class="menu-title">
								<span class="input-title">
									<input type="checkbox" class="ova_check_menu_item" value="1">
									<span class="title">
										<?php echo esc_html( $title ); ?>
									</span>
								</span>
								<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
							</h4>
							<div class="content">
								<ul class="list-menu-item-field">
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
										placeholder=""
										value="<?php echo esc_attr( $title ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
										placeholder=""
										value="<?php echo esc_attr( $url ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title_attr large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_classes large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for="">
										<input
										type="checkbox"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
										value="1" />
										<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
										</label>
									</li>
									<li class="menu-item-field footer">
										<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
										<span class="separator">|</span>
										<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
									</li>
								</ul>
							</div>
						</li>
				<?php $k += 1; endforeach; endif;
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_filter_categories(){
			$response = '';
			$name = isset( $_POST['name'] ) ? sanitize_text_field( $_POST['name'] ) : '';
			$taxonomy = 'category';

			ob_start();


			if ( ! empty( $name ) ) {
				$terms = ova_megamenu_get_terms_by_name( $name, $taxonomy );
				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						?>
						<li class="item <?php echo esc_attr( $taxonomy ); ?>-item">
							<label for="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>" value="<?php echo esc_attr( $term->term_id ); ?>">
								<?php echo esc_html( $term->name ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php esc_html_e( 'Category not found.', 'ova-megamenu' ); ?></p>
					<?php
				}

			} else {
				$terms = ova_megamenu_get_categories_default();
				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						?>
						<li class="item <?php echo esc_attr( $taxonomy ); ?>-item">
							<label for="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>" value="<?php echo esc_attr( $term->term_id ); ?>">
								<?php echo esc_html( $term->name ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php esc_html_e( 'Category not found.', 'ova-megamenu' ); ?></p>
					<?php
				}
			}

			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_custom_tax_item(){
			$response = '';
			$term_ids = isset( $_POST['term_ids'] ) ? $_POST['term_ids'] : array();
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			ob_start();
				if ( ! empty( $term_ids ) ):
					$k = (int) $menu_item;
					foreach ( $term_ids as $term_id ) :
						$category = get_term( $term_id );
						$title = $category->name;
						$url = get_term_link( $category );
						?>
						<li class="menu-item">
							<h4 class="menu-title">
								<span class="input-title">
									<input type="checkbox" class="ova_check_menu_item" value="1">
									<span class="title">
										<?php echo esc_html( $title ); ?>
									</span>
								</span>
								<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
							</h4>
							<div class="content">
								<ul class="list-menu-item-field">
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
										placeholder=""
										value="<?php echo esc_attr( $title ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
										placeholder=""
										value="<?php echo esc_attr( $url ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title_attr large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_classes large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for="">
										<input
										type="checkbox"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
										value="1" />
										<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
										</label>
									</li>
									<li class="menu-item-field footer">
										<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
										<span class="separator">|</span>
										<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
									</li>
								</ul>
							</div>
						</li>
				<?php $k += 1; endforeach; endif;
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_filter_custom_taxonomies(){
			$response = '';
			$name = isset( $_POST['name'] ) ? sanitize_text_field( $_POST['name'] ) : '';
			$taxonomy = isset( $_POST['taxonomy'] ) ? sanitize_text_field( $_POST['taxonomy'] ) : '';
			$taxonomy_obj = get_taxonomy( $taxonomy );
			$label = $taxonomy_obj->labels->singular_name;
			ob_start();


			if ( ! empty( $name ) ) {
				$terms = ova_megamenu_get_terms_by_name( $name, $taxonomy );
				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						?>
						<li class="item <?php echo esc_attr( $taxonomy ); ?>-item">
							<label for="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>" value="<?php echo esc_attr( $term->term_id ); ?>">
								<?php echo esc_html( $term->name ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php echo sprintf( esc_html__( '%s not found.', 'ova-megamenu' ), $label ) ?></p>
					<?php
				}

			} else {
				$terms = ova_megamenu_get_custom_taxonomies_default( $taxonomy );
				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						?>
						<li class="item <?php echo esc_attr( $taxonomy ); ?>-item">
							<label for="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $taxonomy.'-'.$term->term_id  ); ?>" value="<?php echo esc_attr( $term->term_id ); ?>">
								<?php echo esc_html( $term->name ); ?>
							</label>
						</li>
						<?php
					}
				} else {
					?>
					<p><?php echo sprintf( esc_html__( '%s not found.', 'ova-megamenu' ), $label ) ?></p>
					<?php
				}
			}

			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_filter_custom_post_type(){
			$response = '';
			$title = isset( $_POST['title'] ) ? sanitize_text_field( $_POST['title'] ) : '';
			$post_type = isset( $_POST['post_type'] ) ? sanitize_text_field( $_POST['post_type'] ) : '';
			$post_type_obj = get_post_type_object( $post_type );

			$label = $post_type_obj->labels->singular_name;
			ob_start();


			if ( ! empty( $title ) ) {
				$custom_posts = ova_megamenu_get_custom_post_by_title( $title, $post_type );
				if ( $custom_posts && $custom_posts->have_posts()  ) {
					while ( $custom_posts->have_posts() ) {
						$custom_posts->the_post();
						$id = get_the_ID();
						$title = get_the_title();
						?>
						<li class="item <?php echo esc_attr( $post_type ); ?>-item">
							<label for="<?php echo esc_attr( $post_type.'-'.$id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $post_type.'-'.$id  ); ?>" value="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html( $title ); ?>
							</label>
						</li>
						<?php
					}
					wp_reset_postdata();
				} else {
					?>
					<p><?php echo sprintf( esc_html__( '%s not found.', 'ova-megamenu' ), $label ) ?></p>
					<?php
				}

			} else {
				$custom_posts = ova_megamenu_get_custom_post_type_default( $post_type );
				if ( $custom_posts && $custom_posts->have_posts() ) {
					while ( $custom_posts->have_posts() ) {
						$custom_posts->the_post();
						$id = get_the_ID();
						$title = get_the_title();
						?>
						<li class="item <?php echo esc_attr( $post_type ); ?>-item">
							<label for="<?php echo esc_attr( $post_type.'-'.$id  ); ?>">
								<input type="checkbox" id="<?php echo esc_attr( $post_type.'-'.$id  ); ?>" value="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html( $title ); ?>
							</label>
						</li>
						<?php
					}
					wp_reset_postdata();
				} else {
					?>
					<p><?php echo sprintf( esc_html__( '%s not found.', 'ova-megamenu' ), $label ) ?></p>
					<?php
				}
			}

			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_add_custom_post_item(){
			$response = '';
			$post_ids = isset( $_POST['post_ids'] ) ? $_POST['post_ids'] : array();
			$row = isset( $_POST['row'] ) ? sanitize_text_field( $_POST['row'] ) : '';
			$column = isset( $_POST['column'] ) ? sanitize_text_field( $_POST['column'] ) : '';
			$widget = isset( $_POST['widget'] ) ? sanitize_text_field( $_POST['widget'] ) : '';
			$menu_item = isset( $_POST['menu_item'] ) ? sanitize_text_field( $_POST['menu_item'] ) : '';
			$this_tab = isset( $_POST['this_tab'] ) ? sanitize_text_field( $_POST['this_tab'] ) : '';
			ob_start();
				if ( ! empty( $post_ids ) ):
					$k = (int) $menu_item;
					foreach ( $post_ids as $post_id ) :
						$title = get_the_title( $post_id );
						$url = get_permalink( $post_id );
						?>
						<li class="menu-item">
							<h4 class="menu-title">
								<span class="input-title">
									<input type="checkbox" class="ova_check_menu_item" value="1">
									<span class="title">
										<?php echo esc_html( $title ); ?>
									</span>
								</span>
								<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
							</h4>
							<div class="content">
								<ul class="list-menu-item-field">
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][label]"
										placeholder=""
										value="<?php echo esc_attr( $title ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][url]"
										placeholder=""
										value="<?php echo esc_attr( $url ); ?>" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_title_attr large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][title]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
										<input
										type="text"
										class="menu_item_classes large-text"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][class]"
										placeholder=""
										value="" />
									</li>
									<li class="menu-item-field">
										<label for="">
										<input
										type="checkbox"
										name="ova_megamenu_<?php echo $this_tab; ?>[<?php echo $row; ?>][<?php echo $column; ?>][widget][<?php echo $widget; ?>][menu_item][<?php echo $k; ?>][target]"
										value="1" />
										<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
										</label>
									</li>
									<li class="menu-item-field footer">
										<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
										<span class="separator">|</span>
										<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
									</li>
								</ul>
							</div>
						</li>
				<?php $k += 1; endforeach; endif;
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_select_font(){
			$response = '';
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_megamenu_select_font' ) ) {
				echo $response;
				wp_die();
			}
			$font 			= isset( $_POST['font'] ) ? $_POST['font'] : '';
			$font_weight 	= isset( $_POST['font_weight'] ) ? $_POST['font_weight'] : '';
			ob_start();
			if ( ! empty( $font ) ) {
				$google_fonts = new OVAM_Google_Fonts();
				$font_weights = $google_fonts->get_font_weight( $font );

				if ( ! empty( $font_weights ) ) {
					foreach ( $font_weights as $key => $val ) {
						?>
						<option value="<?php echo esc_attr( $val ); ?>" <?php selected( $font_weight, $val ); ?>><?php echo esc_html( $val ); ?></option>
						<?php
					}
				}
			
			} else {
				?>
				<option value="normal" <?php selected( $font_weight, 'normal' ); ?> ><?php echo esc_html( 'normal' ); ?></option>
				<option value="bold" <?php selected( $font_weight, 'bold' ); ?> ><?php echo esc_html( 'bold' ); ?></option>
				<?php
			}
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_export_settings(){
			$response = array();

			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_export_settings' ) ) {
				echo json_encode( $response );
				wp_die();
			}
			$options = get_option( 'ova_megamenu', array() );
			$response = array(
				'id' => password_hash('ova_megamenu',PASSWORD_DEFAULT),
				'version' => OVA_MEGAMENU_VERSION,
				'data' => $options,
			);
			ob_start();
			?>
			<div class="ova_export_modal">
				<p class="description"><?php esc_html_e( 'Copy the data below then paste into import settings.', $domain = 'default' ) ?></p>
				<textarea cols="30" rows=10 class="ova_export_content"><?php echo json_encode( $response );?></textarea>
			</div>
			<?php
			$html = ob_get_clean();
			echo $html;
			wp_die();
		}

		public function ova_megamenu_import_popup(){
			ob_start();
			?>
			<div class="ova_import_modal">
				<textarea cols="30" rows=10 class="ova_import_content" placeholder="<?php esc_attr_e( 'Insert data here', 'ova-megamenu' ); ?>"></textarea>
			</div>
			<?php
			$response = ob_get_clean();
			echo $response;
			wp_die();
		}

		public function ova_megamenu_import_settings(){
			$response = [];
			$response['status'] = 'success';

			$response['mess'] = esc_html__( 'Data import was successful.', 'ova-megamenu' );
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_import_settings' ) ) {
				$response['status'] = 'error';
				$response['mess'] = esc_html__( 'Nonce Verification Failed.', 'ova-megamenu' );
				echo json_encode( $response );
				wp_die();
			}

			$data 	= isset( $_POST['data'] ) 	? ova_recursive_sanitize_text_field( $_POST['data'] ) : [];
			$id 	= isset( $data['id'] ) 		? $data['id'] : '';
			$opts 	= isset( $data['data'] ) 	? $data['data'] : [];

			if ( ! password_verify('ova_megamenu', $id ) ) {
				$response['status'] = 'error';
				$response['mess'] = esc_html__( 'Data is invalid.', 'ova-megamenu' );
			} else {
				if ( ! empty( $opts ) ) {
					update_option( 'ova_megamenu', $opts );
				}
			}

			echo json_encode( $response );
			wp_die();
		}

		public function ova_megamenu_reset_settings(){
			$response = [];
			$response['status'] = 'success';

			$response['mess'] = esc_html__( 'Reset data successfully.', 'ova-megamenu' );
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_reset_settings' ) ) {
				$response['status'] = 'error';
				$response['mess'] = esc_html__( 'Nonce Verification Failed.', 'ova-megamenu' );
				echo json_encode( $response );
				wp_die();
			}

			update_option( 'ova_megamenu', '' );

			echo json_encode( $response );
			wp_die();
		}

		public function ova_megamenu_copy_update(){
			$response = '';
			if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'ova_megamenu_copy_update' ) ) {
				echo $response;
				wp_die();
			}
			$post_id 	= isset( $_POST['id'] ) ? sanitize_text_field( $_POST['id'] ) : '';
			$tab_name 	= isset( $_POST['tab'] ) ? sanitize_text_field( $_POST['tab'] ) : '';
			$copy_from 	= isset( $_POST['copy_from'] ) ? sanitize_text_field( $_POST['copy_from'] ) : '';

			if ( ! empty( $tab_name ) && ! empty( $copy_from ) && ! empty( $post_id ) ) {

				$ova_megamenu_desktop 	= get_post_meta( $post_id, 'ova_megamenu_desktop', true );
				$ova_megamenu_tablet 	= get_post_meta( $post_id, 'ova_megamenu_tablet', true );
				$ova_megamenu_mobile 	= get_post_meta( $post_id, 'ova_megamenu_mobile', true );

				switch ( $tab_name ) {
					case 'desktop':

						switch ( $copy_from ) {
							case 'tablet':
								$ova_megamenu_desktop = $ova_megamenu_tablet;
								break;
							case 'mobile':
								$ova_megamenu_desktop = $ova_megamenu_mobile;
								break;
							default:
								break;
						}
						update_post_meta( $post_id, 'ova_megamenu_desktop', $ova_megamenu_desktop );

						break;
					case 'tablet':
						
						switch ( $copy_from ) {
							case 'desktop':
								$ova_megamenu_tablet = $ova_megamenu_desktop;
								break;
							case 'mobile':
								$ova_megamenu_tablet = $ova_megamenu_mobile;
								break;
							default:
								break;
						}
						update_post_meta( $post_id, 'ova_megamenu_tablet', $ova_megamenu_tablet );

						break;
					case 'mobile':
						
						switch ( $copy_from ) {
							case 'desktop':
								$ova_megamenu_mobile = $ova_megamenu_desktop;
								break;
							case 'tablet':
								$ova_megamenu_mobile = $ova_megamenu_tablet;
								break;
							default:
								break;
						}

						update_post_meta( $post_id, 'ova_megamenu_mobile', $ova_megamenu_mobile );

						break;
					default:
						break;
				}

				$ova_megamenu_tab_current = get_post_meta( $post_id, 'ova_megamenu_'.$tab_name, true );
				ob_start();
				ova_megamenu_get_tab_content( $ova_megamenu_tab_current, $tab_name );
				$response = ob_get_clean();
			}
			echo $response;
			wp_die();
		}

	}

	new OVAM_Admin_Ajax();
}