<?php

if ( !defined( 'ABSPATH' ) ) exit();

if ( !class_exists('OVAM_Settings' ) ) {
    class OVAM_Settings {

        protected $sections;

        protected $border_style;

        protected $text_transform_opts;

        protected $text_decoration_opts;

        protected $text_align_opts;

        protected $google_fonts;

        protected $page_id = 'ova_megamenu';

        function __construct() {

            $this->sections = $this->get_settings_sections();
            $this->border_style = $this->get_border_style();
            $this->text_transform_opts = $this->get_text_transform_opts();
            $this->text_decoration_opts = $this->get_text_decoration_opts();
            $this->text_align_opts = $this->get_text_align_opts();
            $this->google_fonts = new OVAM_Google_Fonts();

            add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
            add_action( 'admin_init', array($this, 'admin_init') );
            add_action( 'admin_menu', array($this, 'admin_menu') );
        }

        function admin_enqueue_scripts() {
            wp_enqueue_style( 'wp-color-picker' );
            wp_enqueue_media();
            wp_enqueue_script( 'jquery' );
            wp_register_script( 'wp-color-picker-alpha', OVA_MEGAMENU_PLUGIN_URI.'assets/libs/wp-color-picker-alpha.min.js', array('wp-color-picker'), false, true );
            wp_enqueue_script( 'wp-color-picker-alpha' );
            wp_enqueue_code_editor(array(
                'type'       => 'text/css',
            ));
        }

        function admin_init() {
            $page_id = $this->page_id;
            $sections = $this->sections;

            if ( false == get_option( $page_id ) ) {
                add_option( $page_id );
            }

            $settings_fields = $this->get_settings_fields();

            foreach ( $settings_fields as $section => $field ) {
                foreach ( $field as $option ) {

                    $name = $option['name'];
                    $type = isset( $option['type'] ) ? $option['type'] : 'text';
                    $label = isset( $option['label'] ) ? $option['label'] : '';
                    $callback = isset( $option['callback'] ) ? $option['callback'] : array( $this, 'callback_' . $type );

                    $args = array(
                        'page_id'           => $page_id,
                        'id'                => $name,
                        'class'             => isset( $option['class'] ) ? $option['class'] : $name,
                        'label_for'         => "{$page_id}[{$section}][{$name}]",
                        'desc'              => isset( $option['desc'] ) ? $option['desc'] : '',
                        'name'              => $label,
                        'section'           => $section,
                        'size'              => isset( $option['size'] ) ? $option['size'] : null,
                        'options'           => isset( $option['options'] ) ? $option['options'] : '',
                        'std'               => isset( $option['default'] ) ? $option['default'] : '',
                        'sanitize_callback' => isset( $option['sanitize_callback'] ) ? $option['sanitize_callback'] : '',
                        'type'              => $type,
                        'placeholder'       => isset( $option['placeholder'] ) ? $option['placeholder'] : '',
                        'min'               => isset( $option['min'] ) ? $option['min'] : '',
                        'max'               => isset( $option['max'] ) ? $option['max'] : '',
                        'step'              => isset( $option['step'] ) ? $option['step'] : '',
                    );

                    add_settings_field( "{$page_id}[{$section}][{$name}]", $label, $callback, $page_id, $section, $args );

                }
            }

            // creates our settings in the options table
            register_setting( $page_id, $page_id, array( $this, 'sanitize_options' ) );
        }

        function admin_menu() {

            add_submenu_page( 'edit.php?post_type=ova-megamenu', esc_html__( 'Settings', 'ova-megamenu' ), esc_html__( 'Settings', 'ova-megamenu' ), 'manage_options', 'ova_megamenu_settings', array($this, 'plugin_page') );
        }

        function get_settings_sections(){

            $sections = array(
                array(
                    'id' => 'general',
                    'title' => esc_html__( 'General Settings', 'ova-megamenu' ),
                ),
                array(
                    'id' => 'mega_menus',
                    'title' => esc_html__( 'Mega Menus', 'ova-megamenu' ),
                ),
                array(
                    'id' => 'advanced',
                    'title' => esc_html__( 'Advanced', 'ova-megamenu' ),
                ),
            );

            return $sections;
        }

        function get_settings_fields() {
            $settings_fields = array(
                'general' => array(
                    array(
                        'name'              => 'breakpoint',
                        'label'             => esc_html__( 'Responsive Breakpoint', 'ova-megamenu' ),
                        'type'              => 'breakpoint',
                        'default'           => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name'    => 'position',
                        'label'   => esc_html__( 'Position', 'ova-megamenu' ),
                        'type'    => 'radio',
                        'default' => 'center',
                        'options' => array(
                            'left' => esc_html__( 'Left', 'ova-megamenu' ),
                            'center'  => esc_html__( 'Center', 'ova-megamenu' ),
                            'right'  => esc_html__( 'Right', 'ova-megamenu' ),
                        ),
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                        'desc' => esc_html__( 'Position where mega menu is displayed relative to menu item in nav menu. Applies only to width style which is container.', 'ova-megamenu' ),
                    ),
                    array(
                        'name'              => 'z_index',
                        'label'             => esc_html__( 'Z Index', 'ova-megamenu' ),
                        'step'              => '1',
                        'type'              => 'number',
                        'default'           => '999',
                        'sanitize_callback' => 'floatval'
                    ),
                    array(
                        'name' => 'padding',
                        'label' => esc_html__( 'Padding', 'ova-megamenu' ),
                        'type' => 'dimensions',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'background',
                        'label' => esc_html__( 'Background Color', 'ova-megamenu' ),
                        'type' => 'background_color',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'border',
                        'label' => esc_html__( 'Border', 'ova-megamenu' ),
                        'type' => 'border',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'border_radius',
                        'label' => esc_html__( 'Border Radius', 'ova-megamenu' ),
                        'type' => 'border_radius',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    
                    array(
                        'name' => 'box_shadow',
                        'label' => esc_html__( 'Box Shadow', 'ova-megamenu' ),
                        'type' => 'box_shadow',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                ),
                'mega_menus' => array(
                    array(
                        'name' => 'rows',
                        'label' => esc_html__( 'Rows', 'ova-megamenu' ),
                        'type' => 'row_style',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'columns',
                        'label' => esc_html__( 'Columns', 'ova-megamenu' ),
                        'type' => 'column_style',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'widget',
                        'label' => esc_html__( 'Widgets', 'ova-megamenu' ),
                        'type' => 'text_style',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'heading',
                        'label' => esc_html__( 'Heading', 'ova-megamenu' ),
                        'type' => 'heading_style',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                    array(
                        'name' => 'menu_item',
                        'label' => esc_html__( 'Menu Item', 'ova-megamenu' ),
                        'type' => 'menu_item',
                        'default' => '',
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                    ),
                ),
                'advanced' => array(
                    array(
                        'name' => 'custom_style',
                        'label' => esc_html__( 'Custom Style', 'ova-megamenu' ),
                        'type' => 'code_editor',
                        'default' => '',
                        'sanitize_callback' => 'ova_megamenu_strip_all_tags',
                    ),
                    array(
                        'name'    => 'css_output',
                        'label'   => esc_html__( 'Css Output', 'ova-megamenu' ),
                        'type'    => 'select',
                        'default' => 'internal',
                        'options' => array(
                            'internal' => esc_html__( 'Internal', 'ova-megamenu' ),
                            'external'  => esc_html__( 'External', 'ova-megamenu' ),
                        ),
                        'sanitize_callback' => 'ova_recursive_sanitize_text_field',
                        'desc' => esc_html__( 'Internal: The setup css code will be inserted between the opening and closing head tags of the website/ External: The setup css code will be located in the ova-megamenu-style.css file in the wp-content/upload directory.', 'ova-megamenu' ),
                    ),
                ),
            );

            return $settings_fields;
        }


        function plugin_page() {
            $sections = $this->sections;
            $page_id = $this->page_id;
            ?>

            <div class="ova-megamenu-wrap">
                
                <h2 class="nav-tab-wrapper">
                    <?php
                    foreach ( $sections as $key => $section ): ?>
                        <a href="#<?php echo esc_attr( $section['id'] ); ?>" data-id="<?php echo esc_attr( $section['id'] ); ?>" class="nav-tab"><?php echo esc_html( $section['title'] ); ?></a>
                    <?php endforeach; ?>
                </h2>
                

                <form method="POST" action="options.php" class="ova_megamenu_settings_form">
                <?php
                    settings_fields( $page_id );
                    foreach ( $sections as $key => $section ){
                        ?>
                        <div class="tabs-content" id="<?php echo esc_attr( $section['id'] ); ?>">
                            <table class="form-table" role="presentation">
                                <?php
                                    do_settings_fields( $page_id, $section['id'] );
                                ?>
                            </table>
                        </div>
                        <?php
                    }
                    
                ?>
                <div style="padding-left: 10px">
                    <?php submit_button(); ?>
                </div>
                </form>



            </div>
            <script type="text/javascript">
                (function($) {
                    $(function(){
                        $('.wp-color-picker-field').wpColorPicker();
                        $('.wpsa-browse').on('click', function (event) {
                            event.preventDefault();

                            var self = $(this);

                            // Create the media frame.
                            var file_frame = wp.media.frames.file_frame = wp.media({
                                title: self.data('uploader_title'),
                                button: {
                                    text: self.data('uploader_button_text'),
                                },
                                multiple: false
                            });

                            file_frame.on('select', function () {
                                attachment = file_frame.state().get('selection').first().toJSON();
                                self.prev('.wpsa-url').val(attachment.url).change();
                            });

                            // Finally, open the modal
                            file_frame.open();
                        });
                        wp.codeEditor.initialize( $('#code_editor') );
                    });
                })(jQuery);
                
            </script>
            <?php
        }

            /**
         * Get field description for display
         *
         * @param array   $args settings field args
         */
        public function get_field_description( $args ) {
            if ( ! empty( $args['desc'] ) ) {
                $desc = sprintf( '<p class="description">%s</p>', $args['desc'] );
            } else {
                $desc = '';
            }

            return $desc;
        }

        /**
         * Displays a text field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_text( $args ) {

            $value       = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size        = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';
            $type        = isset( $args['type'] ) ? $args['type'] : 'text';
            $placeholder = empty( $args['placeholder'] ) ? '' : ' placeholder="' . $args['placeholder'] . '"';

            $html        = sprintf( '<input type="%1$s" class="%2$s-text" id="%5$s[%3$s][%4$s]" name="%3$s[%4$s]" value="%5$s"%6$s/>', $type, $size, $args['section'], $args['id'], $value, $placeholder, $args['page_id'] );
            $html       .= $this->get_field_description( $args );

            echo $html;
        }

        /**
         * Displays a url field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_url( $args ) {
            $this->callback_text( $args );
        }

        function get_border_style(){
            $style_opts = array(
                'none'      => esc_html__( 'None', 'ova-megamenu' ),
                'dotted'    => esc_html__( 'Dotted', 'ova-megamenu' ),
                'dashed'    => esc_html__( 'Dashed', 'ova-megamenu' ),
                'solid'     => esc_html__( 'Solid', 'ova-megamenu' ),
            );
            return $style_opts;
        }

        function get_text_transform_opts(){
            $text_transform_opts = array(
                'none'          => esc_html__( 'None', 'ova-megamenu' ),
                'uppercase'     => esc_html__( 'Uppercase', 'ova-megamenu' ),
                'lowercase'     => esc_html__( 'Lowercase', 'ova-megamenu' ),
                'capitalize'    => esc_html__( 'Capitalize', 'ova-megamenu' ),
            );
            return $text_transform_opts;
        }

        function get_text_decoration_opts(){
            $text_decoration_opts = array(
                'none'      => esc_html__( 'None', 'ova-megamenu' ),
                'underline' => esc_html__( 'Underline', 'ova-megamenu' ),
            );
            return $text_decoration_opts;
        }

        function get_text_align_opts(){
            $text_align_opts = array(
                'left'      => esc_html__( 'Left', 'ova-megamenu' ),
                'right'     => esc_html__( 'Right', 'ova-megamenu' ),
                'center'    => esc_html__( 'Center', 'ova-megamenu' ),
            );
            return $text_align_opts;
        }

        function callback_ova_item_number( $args, $label, $key, $value ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key ,$label );
                $html .= sprintf( '<input type="number" class="small-text" id="%1$s[%2$s][%3$s]%4$s" name="%1$s[%2$s][%3$s]%4$s" min="0" step="1" value="%5$s"/>', $args['page_id'], $args['section'], $args['id'], $key ,$value );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_select( $args, $label, $key, $value, $options ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key, $label );
                $html .= sprintf( '<select class="regular" name="%1$s[%2$s][%3$s]%4$s" id="%1$s[%2$s][%3$s]%4$s">', $args['page_id'] ,$args['section'], $args['id'], $key );

                    foreach ( $options as $k => $val ) {
                        $html .= sprintf( '<option value="%1$s" %2$s>%3$s</option>', $k ,selected( $value, $k, false ), $val );
                    }

                $html .= sprintf( '</select>' );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_select_background_direction( $args, $label, $key, $value ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key, $label );
                $html .= sprintf( '<select class="regular" name="%1$s[%2$s][%3$s]%4$s" id="%1$s[%2$s][%3$s]%4$s">', $args['page_id'] ,$args['section'], $args['id'], $key );
                    $html .= sprintf( '<option value="" %1$s>%2$s</option>', selected( $value, '', false ) ,esc_html__( 'To bottom', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to right" %1$s>%2$s</option>', selected( $value, 'to right', false ) ,esc_html__( 'To right', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to left" %1$s>%2$s</option>', selected( $value, 'to left', false ) ,esc_html__( 'To left', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to bottom right" %1$s>%2$s</option>', selected( $value, 'to bottom right', false ) ,esc_html__( 'To bottom right', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to bottom left" %1$s>%2$s</option>', selected( $value, 'to bottom left', false ) ,esc_html__( 'To bottom left', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to top right" %1$s>%2$s</option>', selected( $value, 'to top right', false ) ,esc_html__( 'To top right', 'ova-megamenu' ) );
                    $html .= sprintf( '<option value="to top left" %1$s>%2$s</option>', selected( $value, 'to top left', false ) ,esc_html__( 'To top left', 'ova-megamenu' ) );
                    
                $html .= sprintf( '</select>' );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_color( $args, $label, $key, $value ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key , $label );
                $html .= sprintf( '<input type="text" class="regular-text wp-color-picker-field" id="%1$s[%2$s][%3$s]%4$s" name="%1$s[%2$s][%3$s]%4$s" value="%5$s" data-default-color="" data-alpha-enabled="true" />', $args['page_id'] ,$args['section'], $args['id'], $key , $value );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_checkbox( $args, $label, $key, $value ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key, $label );
                $html .= sprintf( '<input type="checkbox" class="checkbox" id="%1$s[%2$s][%3$s]%4$s" name="%1$s[%2$s][%3$s]%4$s" %5$s value="1"/>',$args['page_id'] ,$args['section'], $args['id'], $key ,checked( $value, '1', false ) );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_select_font( $args, $label, $key, $value, $list_font ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key, $label );
                $html .= sprintf( '<select class="regular select_font" data-nonce="%5$s" name="%1$s[%2$s][%3$s]%4$s" id="%1$s[%2$s][%3$s]%4$s">', $args['page_id'] ,$args['section'], $args['id'], $key ,wp_create_nonce( 'ova_megamenu_select_font' ) );
                    $html .= sprintf( '<option value="" %1$s>%2$s</option>', selected( $value, 'Theme default', false ) ,esc_html__( 'Theme default', 'ova-megamenu' ) );
                    foreach ( $list_font as $item ) {
                        $html .= sprintf( '<option value="%2$s" %1$s>%2$s</option>', selected( $value, $item, false ) , esc_html( $item ) );
                    }
                $html .= sprintf( '</select>' );
            $html .= '</li>';

            return $html;
        }

        function callback_ova_item_select_font_weight( $args, $label, $key, $value ){
            $html = '';

            $html .= '<li class="ova-item">';
                $html .= sprintf( '<label class="ova-label" for="%1$s[%2$s][%3$s]%4$s">%5$s</label>', $args['page_id'] ,$args['section'], $args['id'], $key ,$label );
                $html .= sprintf( '<select class="regular select_font_weight" data-value="%5$s" name="%1$s[%2$s][%3$s]%4$s" id="%1$s[%2$s][%3$s]%4$s">', $args['page_id'] ,$args['section'], $args['id'], $key ,$value );
                $html .= sprintf( '</select>' );
            $html .= '</li>';

            return $html;
        }

        function callback_border( $args ){

            $option = $this->get_option( $args['id'], $args['section'], array() );

            $color  = isset( $option['color'] )     ? $option['color']  : '';
            $top    = isset( $option['top'] )       ? $option['top']    : '';
            $right  = isset( $option['right'] )     ? $option['right']  : '';
            $bottom = isset( $option['bottom'] )    ? $option['bottom'] : '';
            $left   = isset( $option['left'] )      ? $option['left']   : '';
            $style  = isset( $option['style'] )     ? $option['style']  : '';

            $style_opts = $this->border_style;

            $html = '<ul class="ova-list">';

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[top]', $top );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[right]', $right );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[bottom]', $bottom );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[left]', $left );

                $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[style]', $style, $style_opts );

                $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

            $html .= '</ul>';

            echo $html;         

        }

        function callback_dimensions( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $left   = isset( $option['left'] )      ? $option['left']   : '';
            $top    = isset( $option['top'] )       ? $option['top']    : '';
            $right  = isset( $option['right'] )     ? $option['right']  : '';
            $bottom = isset( $option['bottom'] )    ? $option['bottom'] : '';

            $html = '<ul class="ova-list">';

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[left]', $left );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[top]', $top );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[right]', $right );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[bottom]', $bottom );

            $html .= '</ul>';

            echo $html; 
        }

        function callback_box_shadow( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $horizontal = isset( $option['horizontal'] )    ? $option['horizontal'] : '';
            $vertical   = isset( $option['vertical'] )      ? $option['vertical']   : '';
            $blur       = isset( $option['blur'] )          ? $option['blur']       : '';
            $spread     = isset( $option['spread'] )        ? $option['spread']     : '';
            $color      = isset( $option['color'] )         ? $option['color']      : '';

            $html = '<ul class="ova-list">';


                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[horizontal]', $horizontal );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[vertical]', $vertical );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[blur]', $blur );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[spread]', $spread );

                $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

            $html .= '</ul>';

            echo $html;
        }

        function callback_border_radius( $args ){

            $option = $this->get_option( $args['id'], $args['section'], array() );

            $top_left       = isset( $option['top-left'] )      ? $option['top-left']       : '';
            $top_right      = isset( $option['top-right'] )     ? $option['top-right']      : '';
            $bottom_right   = isset( $option['bottom-right'] )  ? $option['bottom-right']   : '';
            $bottom_left    = isset( $option['bottom-left'] )   ? $option['bottom-left']    : '';

            $html = '<ul class="ova-list">';


                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[top-left]', $top_left );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[top-right]', $top_right );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[bottom-right]', $bottom_right );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[bottom-left]', $bottom_left );

            $html .= '</ul>';

            echo $html;
        }

        function callback_separator( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $enable = isset( $option['enable'] )    ? $option['enable'] : '';
            $color  = isset( $option['color'] )     ? $option['color']  : '';
            $width  = isset( $option['width'] )     ? $option['width']  : '';

            $html = '<ul class="ova-list">';

                $html .= $this->callback_ova_item_checkbox( $args, esc_html__( 'Enable', 'ova-megamenu' ), '[enable]', $enable );

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Width(px)', 'ova-megamenu' ), '[width]', $width );

                $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

            $html .= '</ul>';

            echo $html;
        }

        function callback_text_style( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $color = isset( $option['color'] ) ? $option['color'] : '';

            $background_direction   = isset( $option['background']['direction'] )   ? $option['background']['direction']    : '';
            $background_from        = isset( $option['background']['from'] )        ? $option['background']['from']         : '';
            $background_to          = isset( $option['background']['to'] )          ? $option['background']['to']           : '';
            
            $size               = isset( $option['size'] )          ? $option['size']           : '';
            $line_height        = isset( $option['line_height'] )   ? $option['line_height']    : '';
            $font_family        = isset( $option['font_family'] )   ? $option['font_family']    : '';
            $font_weight        = isset( $option['font_weight'] )   ? $option['font_weight']    : '';
            $text_align         = isset( $option['align'] )         ? $option['align']          : '';
            $text_transform     = isset( $option['transform'] )     ? $option['transform']      : '';
            $text_decoration    = isset( $option['decoration'] )    ? $option['decoration']     : '';

            $padding_left   = isset( $option['padding']['left'] )   ? $option['padding']['left']    : '';
            $padding_top    = isset( $option['padding']['top'] )    ? $option['padding']['top']     : '';
            $padding_right  = isset( $option['padding']['right'] )  ? $option['padding']['right']   : '';
            $padding_bottom = isset( $option['padding']['bottom'] ) ? $option['padding']['bottom']  : '';

            $margin_left    = isset( $option['margin']['left'] )    ? $option['margin']['left']     : '30';
            $margin_top     = isset( $option['margin']['top'] )     ? $option['margin']['top']      : '30';
            $margin_right   = isset( $option['margin']['right'] )   ? $option['margin']['right']    : '30';
            $margin_bottom  = isset( $option['margin']['bottom'] )  ? $option['margin']['bottom']   : '30';

            $border_top     = isset( $option['border']['top'] )     ? $option['border']['top']      : '';
            $border_right   = isset( $option['border']['right'] )   ? $option['border']['right']    : '';
            $border_bottom  = isset( $option['border']['bottom'] )  ? $option['border']['bottom']   : '';
            $border_left    = isset( $option['border']['left'] )    ? $option['border']['left']     : '';
            $border_style   = isset( $option['border']['style'] )   ? $option['border']['style']    : '';
            $border_color   = isset( $option['border']['color'] )   ? $option['border']['color']    : '';

            $border_radius_top_left     = isset( $option['border_radius']['top-left'] )     ? $option['border_radius']['top-left'] : '';
            $border_radius_top_right    = isset( $option['border_radius']['top-right'] )    ? $option['border_radius']['top-right'] : '';
            $border_radius_bottom_right = isset( $option['border_radius']['bottom-right'] ) ? $option['border_radius']['bottom-right'] : '';
            $border_radius_bottom_left  = isset( $option['border_radius']['bottom-left'] )  ? $option['border_radius']['bottom-left'] : '';

            $box_shadow_color       = isset( $option['box_shadow']['color'] )       ? $option['box_shadow']['color'] : '';
            $box_shadow_vertical    = isset( $option['box_shadow']['vertical'] )    ? $option['box_shadow']['vertical'] : '';
            $box_shadow_horizontal  = isset( $option['box_shadow']['horizontal'] )  ? $option['box_shadow']['horizontal'] : '';
            $box_shadow_blur        = isset( $option['box_shadow']['blur'] )        ? $option['box_shadow']['blur'] : '';
            $box_shadow_spread      = isset( $option['box_shadow']['spread'] )      ? $option['box_shadow']['spread'] : '';

            $google_fonts = $this->google_fonts;

            $list_font = $google_fonts->get_font_list_family();

            $text_transform_opts = $this->text_transform_opts;

            $text_decoration_opts = $this->text_decoration_opts;

            $text_align_opts = $this->text_align_opts;

            $border_style_opts = $this->border_style;

            $html = '<ul class="ova-panels">';

                /* font */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Font', 'ova-megamenu' ).'</h5>';
                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Size(px)', 'ova-megamenu' ), '[size]', $size );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Line Height(px)', 'ova-megamenu' ), '[line_height]', $line_height );

                        $html .= $this->callback_ova_item_select_font( $args, esc_html__( 'Font Family', 'ova-megamenu' ), '[font_family]', $font_family, $list_font );

                        $html .= $this->callback_ova_item_select_font_weight( $args, esc_html__( 'Font Weight', 'ova-megamenu' ), '[font_weight]', $font_weight );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Transform', 'ova-megamenu' ), '[transform]', $text_transform, $text_transform_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Decoration', 'ova-megamenu' ), '[decoration]', $text_decoration, $text_decoration_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Align', 'ova-megamenu' ), '[align]', $text_align, $text_align_opts );

                    $html .= '</ul>';

                $html .= '</li>';

                /* padding */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Padding', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[padding][left]', $padding_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[padding][top]', $padding_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[padding][right]', $padding_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[padding][bottom]', $padding_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* margin */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Margin', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[margin][left]', $margin_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[margin][top]', $margin_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[margin][right]', $margin_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[margin][bottom]', $margin_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="ova-list">';

                            $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background][direction]', $background_direction );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background][from]', $background_from );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background][to]', $background_to );

                        $html .= '</ul>';

                $html .= '</li>';

                /* border */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[border][top]', $border_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[border][right]', $border_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[border][bottom]', $border_bottom );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[border][left]', $border_left );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[border][style]', $border_style, $border_style_opts );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[border][color]', $border_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border radius */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border Radius', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[border_radius][top-left]', $border_radius_top_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[border_radius][top-right]', $border_radius_top_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[border_radius][bottom-right]', $border_radius_bottom_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[border_radius][bottom-left]', $border_radius_bottom_left );

                    $html .= '</ul>';

                $html .= '</li>';

                /* box shadow */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Box Shadow', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[box_shadow][horizontal]', $box_shadow_horizontal );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[box_shadow][vertical]', $box_shadow_vertical );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[box_shadow][blur]', $box_shadow_blur );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[box_shadow][spread]', $box_shadow_spread );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[box_shadow][color]', $box_shadow_color );

                    $html .= '</ul>';

                $html .= '</li>';

            $html .= '</ul>';

            echo $html;
        }

        function callback_heading_style( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $color = isset( $option['color'] ) ? $option['color'] : '';

            $background_direction   = isset( $option['background']['direction'] )   ? $option['background']['direction'] : '';
            $background_from        = isset( $option['background']['from'] )        ? $option['background']['from'] : '';
            $background_to          = isset( $option['background']['to'] )          ? $option['background']['to'] : '';
            
            $size           = isset( $option['size'] )          ? $option['size'] : '';
            $line_height    = isset( $option['line_height'] )   ? $option['line_height'] : '';
            $font_family    = isset( $option['font_family'] )   ? $option['font_family'] : '';
            $font_weight    = isset( $option['font_weight'] )   ? $option['font_weight'] : '';
            $text_align     = isset( $option['align'] )         ? $option['align'] : '';
            $text_transform = isset( $option['transform'] )     ? $option['transform'] : '';
            $text_decoration = isset( $option['decoration'] )   ? $option['decoration'] : '';

            $padding_left   = isset( $option['padding']['left'] )   ? $option['padding']['left'] : '';
            $padding_top    = isset( $option['padding']['top'] )    ? $option['padding']['top'] : '';
            $padding_right  = isset( $option['padding']['right'] )  ? $option['padding']['right'] : '';
            $padding_bottom = isset( $option['padding']['bottom'] ) ? $option['padding']['bottom'] : '';

            $margin_left    = isset( $option['margin']['left'] )    ? $option['margin']['left'] : '';
            $margin_top     = isset( $option['margin']['top'] )     ? $option['margin']['top'] : '';
            $margin_right   = isset( $option['margin']['right'] )   ? $option['margin']['right'] : '';
            $margin_bottom  = isset( $option['margin']['bottom'] )  ? $option['margin']['bottom'] : '';

            $border_top     = isset( $option['border']['top'] )     ? $option['border']['top'] : '';
            $border_right   = isset( $option['border']['right'] )   ? $option['border']['right'] : '';
            $border_bottom  = isset( $option['border']['bottom'] )  ? $option['border']['bottom'] : '';
            $border_left    = isset( $option['border']['left'] )    ? $option['border']['left'] : '';
            $border_style   = isset( $option['border']['style'] )   ? $option['border']['style'] : '';
            $border_color   = isset( $option['border']['color'] )   ? $option['border']['color'] : '';

            $border_radius_top_left     = isset( $option['border_radius']['top-left'] )     ? $option['border_radius']['top-left'] : '';
            $border_radius_top_right    = isset( $option['border_radius']['top-right'] )    ? $option['border_radius']['top-right'] : '';
            $border_radius_bottom_right = isset( $option['border_radius']['bottom-right'] ) ? $option['border_radius']['bottom-right'] : '';
            $border_radius_bottom_left  = isset( $option['border_radius']['bottom-left'] )  ? $option['border_radius']['bottom-left'] : '';

            $box_shadow_color       = isset( $option['box_shadow']['color'] )       ? $option['box_shadow']['color'] : '';
            $box_shadow_vertical    = isset( $option['box_shadow']['vertical'] )    ? $option['box_shadow']['vertical'] : '';
            $box_shadow_horizontal  = isset( $option['box_shadow']['horizontal'] )  ? $option['box_shadow']['horizontal'] : '';
            $box_shadow_blur        = isset( $option['box_shadow']['blur'] )        ? $option['box_shadow']['blur'] : '';
            $box_shadow_spread      = isset( $option['box_shadow']['spread'] )      ? $option['box_shadow']['spread'] : '';

            $heading_line_enable    = isset( $option['line']['enable'] )    ? $option['line']['enable'] : '';
            $heading_line_top       = isset( $option['line']['top'] )       ? $option['line']['top'] : '';
            $heading_line_width     = isset( $option['line']['width'] )     ? $option['line']['width'] : '';
            $heading_line_height    = isset( $option['line']['height'] )    ? $option['line']['height'] : '';
            $heading_line_color     = isset( $option['line']['color'] )     ? $option['line']['color'] : '';

            $google_fonts = $this->google_fonts;

            $list_font = $google_fonts->get_font_list_family();

            $text_transform_opts = $this->text_transform_opts;

            $text_decoration_opts = $this->text_decoration_opts;

            $text_align_opts = $this->text_align_opts;

            $border_style_opts = $this->border_style;

            $html = '<ul class="ova-panels">';

                /* font */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Font', 'ova-megamenu' ).'</h5>';
                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Size(px)', 'ova-megamenu' ), '[size]', $size );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Line Height(px)', 'ova-megamenu' ), '[line_height]', $line_height );

                        $html .= $this->callback_ova_item_select_font( $args, esc_html__( 'Font Family', 'ova-megamenu' ), '[font_family]', $font_family, $list_font );

                        $html .= $this->callback_ova_item_select_font_weight( $args, esc_html__( 'Font Weight', 'ova-megamenu' ), '[font_weight]', $font_weight );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Transform', 'ova-megamenu' ), '[transform]', $text_transform, $text_transform_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Decoration', 'ova-megamenu' ), '[decoration]', $text_decoration, $text_decoration_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Align', 'ova-megamenu' ), '[align]', $text_align, $text_align_opts );

                    $html .= '</ul>';

                $html .= '</li>';

                /* padding */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Padding', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[padding][left]', $padding_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[padding][top]', $padding_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[padding][right]', $padding_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[padding][bottom]', $padding_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* margin */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Margin', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[margin][left]', $margin_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[margin][top]', $margin_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[margin][right]', $margin_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[margin][bottom]', $margin_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="ova-list">';

                            $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background][direction]', $background_direction );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background][from]', $background_from );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background][to]', $background_to );

                        $html .= '</ul>';

                $html .= '</li>';

                /* border */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[border][top]', $border_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[border][right]', $border_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[border][bottom]', $border_bottom );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[border][left]', $border_left );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[border][style]', $border_style, $border_style_opts );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[border][color]', $border_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border radius */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border Radius', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[border_radius][top-left]', $border_radius_top_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[border_radius][top-right]', $border_radius_top_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[border_radius][bottom-right]', $border_radius_bottom_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[border_radius][bottom-left]', $border_radius_bottom_left );

                    $html .= '</ul>';

                $html .= '</li>';

                /* box shadow */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Box Shadow', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[box_shadow][horizontal]', $box_shadow_horizontal );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[box_shadow][vertical]', $box_shadow_vertical );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[box_shadow][blur]', $box_shadow_blur );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[box_shadow][spread]', $box_shadow_spread );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[box_shadow][color]', $box_shadow_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* Heading Line */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Heading Line', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_checkbox( $args, esc_html__( 'Enable', 'ova-megamenu' ), '[line][enable]', $heading_line_enable );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[line][top]', $heading_line_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Width(px)', 'ova-megamenu' ), '[line][width]', $heading_line_width );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Height(px)', 'ova-megamenu' ), '[line][height]', $heading_line_height );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[line][color]', $heading_line_color );

                    $html .= '</ul>';

                $html .= '</li>';

            $html .= '</ul>';

            echo $html;
        }

        function callback_menu_item( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $color          = isset( $option['color'] )     ? $option['color'] : '#000';
            $hover          = isset( $option['hover'] )     ? $option['hover'] : '';
            $active         = isset( $option['active'] )    ? $option['active'] : '';

            $background_direction   = isset( $option['background']['direction'] )   ? $option['background']['direction'] : '';
            $background_from        = isset( $option['background']['from'] )        ? $option['background']['from'] : '';
            $background_to          = isset( $option['background']['to'] )          ? $option['background']['to'] : '';

            $background_hover_direction = isset( $option['background_hover']['direction'] ) ? $option['background_hover']['direction'] : '';
            $background_hover_from = isset( $option['background_hover']['from'] ) ? $option['background_hover']['from'] : '';
            $background_hover_to = isset( $option['background_hover']['to'] ) ? $option['background_hover']['to'] : '';
            
            $background_active_direction = isset( $option['background_active']['direction'] ) ? $option['background_active']['direction'] : '';
            $background_active_from = isset( $option['background_active']['from'] ) ? $option['background_active']['from'] : '';
            $background_active_to = isset( $option['background_active']['to'] ) ? $option['background_active']['to'] : '';

            $size               = isset( $option['size'] )          ? $option['size'] : '';
            $line_height        = isset( $option['line_height'] )   ? $option['line_height'] : '';
            $font_family        = isset( $option['font_family'] )   ? $option['font_family'] : '';
            $font_weight        = isset( $option['font_weight'] )   ? $option['font_weight'] : '';
            $text_align         = isset( $option['align'] )         ? $option['align'] : '';
            $text_transform     = isset( $option['transform'] )     ? $option['transform'] : '';
            $text_decoration    = isset( $option['decoration'] )    ? $option['decoration'] : '';

            $padding_left   = isset( $option['padding']['left'] )   ? $option['padding']['left'] : '';
            $padding_top    = isset( $option['padding']['top'] )    ? $option['padding']['top'] : '';
            $padding_right  = isset( $option['padding']['right'] )  ? $option['padding']['right'] : '';
            $padding_bottom = isset( $option['padding']['bottom'] ) ? $option['padding']['bottom'] : '';

            $margin_left    = isset( $option['margin']['left'] )    ? $option['margin']['left'] : '';
            $margin_top     = isset( $option['margin']['top'] )     ? $option['margin']['top'] : '';
            $margin_right   = isset( $option['margin']['right'] )   ? $option['margin']['right'] : '';
            $margin_bottom  = isset( $option['margin']['bottom'] )  ? $option['margin']['bottom'] : '';

            $border_top     = isset( $option['border']['top'] )     ? $option['border']['top'] : '';
            $border_right   = isset( $option['border']['right'] )   ? $option['border']['right'] : '';
            $border_bottom  = isset( $option['border']['bottom'] )  ? $option['border']['bottom'] : '';
            $border_left    = isset( $option['border']['left'] )    ? $option['border']['left'] : '';
            $border_style   = isset( $option['border']['style'] )   ? $option['border']['style'] : '';
            $border_color   = isset( $option['border']['color'] )   ? $option['border']['color'] : '';

            $border_radius_top_left     = isset( $option['border_radius']['top-left'] )     ? $option['border_radius']['top-left'] : '';
            $border_radius_top_right    = isset( $option['border_radius']['top-right'] )    ? $option['border_radius']['top-right'] : '';
            $border_radius_bottom_right = isset( $option['border_radius']['bottom-right'] ) ? $option['border_radius']['bottom-right'] : '';
            $border_radius_bottom_left = isset( $option['border_radius']['bottom-left'] ) ? $option['border_radius']['bottom-left'] : '';

            $box_shadow_color       = isset( $option['box_shadow']['color'] )       ? $option['box_shadow']['color'] : '';
            $box_shadow_vertical    = isset( $option['box_shadow']['vertical'] )    ? $option['box_shadow']['vertical'] : '';
            $box_shadow_horizontal  = isset( $option['box_shadow']['horizontal'] )  ? $option['box_shadow']['horizontal'] : '';
            $box_shadow_blur        = isset( $option['box_shadow']['blur'] )        ? $option['box_shadow']['blur'] : '';
            $box_shadow_spread      = isset( $option['box_shadow']['spread'] )      ? $option['box_shadow']['spread'] : '';

            $google_fonts = new OVAM_Google_Fonts();

            $list_font = $google_fonts->get_font_list_family();

            $text_transform_opts = $this->text_transform_opts;

            $text_decoration_opts = $this->text_decoration_opts;

            $text_align_opts = $this->text_align_opts;

            $border_style_opts = $this->border_style;

            $html = '<ul class="ova-panels">';

                /* font */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Font', 'ova-megamenu' ).'</h5>';
                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[color]', $color );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Size(px)', 'ova-megamenu' ), '[size]', $size );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Line Height(px)', 'ova-megamenu' ), '[line_height]', $line_height );

                        $html .= $this->callback_ova_item_select_font( $args, esc_html__( 'Font Family', 'ova-megamenu' ), '[font_family]', $font_family, $list_font );

                        $html .= $this->callback_ova_item_select_font_weight( $args, esc_html__( 'Font Weight', 'ova-megamenu' ), '[font_weight]', $font_weight );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Transform', 'ova-megamenu' ), '[transform]', $text_transform, $text_transform_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Decoration', 'ova-megamenu' ), '[decoration]', $text_decoration, $text_decoration_opts );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Align', 'ova-megamenu' ), '[align]', $text_align, $text_align_opts );

                    $html .= '</ul>';

                $html .= '</li>';

                /* Color */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Color', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Hover', 'ova-megamenu' ), '[hover]', $hover );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Active', 'ova-megamenu' ), '[active]', $active );

                    $html .= '</ul>';

                $html .= '</li>';

                /* padding */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Padding', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[padding][left]', $padding_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[padding][top]', $padding_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[padding][right]', $padding_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[padding][bottom]', $padding_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* margin */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Margin', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[margin][left]', $margin_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[margin][top]', $margin_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[margin][right]', $margin_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[margin][bottom]', $margin_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="ova-list">';

                            $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background][direction]', $background_direction );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background][from]', $background_from );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background][to]', $background_to );

                        $html .= '</ul>';

                $html .= '</li>';

                /* background hover */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background Hover', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background_hover][direction]', $background_hover_direction );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background_hover][from]', $background_hover_from );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background_hover][to]', $background_hover_to );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background active */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background Active', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background_active][direction]', $background_active_direction );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background_active][from]', $background_active_from );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background_active][to]', $background_active_to );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[border][top]', $border_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[border][right]', $border_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[border][bottom]', $border_bottom );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[border][left]', $border_left );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[border][style]', $border_style, $border_style_opts );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[border][color]', $border_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border radius */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border Radius', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[border_radius][top-left]', $border_radius_top_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[border_radius][top-right]', $border_radius_top_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[border_radius][bottom-right]', $border_radius_bottom_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[border_radius][bottom-left]', $border_radius_bottom_left );

                    $html .= '</ul>';

                $html .= '</li>';

                /* box shadow */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Box Shadow', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[box_shadow][horizontal]', $box_shadow_horizontal );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[box_shadow][vertical]', $box_shadow_vertical );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[box_shadow][blur]', $box_shadow_blur );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[box_shadow][spread]', $box_shadow_spread );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[box_shadow][color]', $box_shadow_color );

                    $html .= '</ul>';

                $html .= '</li>';

            $html .= '</ul>';

            echo $html;
        }

        function callback_breakpoint( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $desktop_max_width = isset( $option['desktop']['max_width'] ) ? $option['desktop']['max_width'] : '1000';
            $desktop_min_height = isset( $option['desktop']['min_height'] ) ? $option['desktop']['min_height'] : '';
            $desktop_max_height = isset( $option['desktop']['max_height'] ) ? $option['desktop']['max_height'] : '';

            $tablet_max_width = isset( $option['tablet']['max_width'] ) ? $option['tablet']['max_width'] : '900';
            $tablet_min_height = isset( $option['tablet']['min_height'] ) ? $option['tablet']['min_height'] : '';
            $tablet_max_height = isset( $option['tablet']['max_height'] ) ? $option['tablet']['max_height'] : '';

            $mobile_max_width = isset( $option['mobile']['max_width'] ) ? $option['mobile']['max_width'] : '767';
            $mobile_min_height = isset( $option['mobile']['min_height'] ) ? $option['mobile']['min_height'] : '';
            $mobile_max_height = isset( $option['mobile']['max_height'] ) ? $option['mobile']['max_height'] : '';

            $tablet_from   = isset( $option['tablet']['from'] )   ? $option['tablet']['from'] : '768';
            $tablet_to   = isset( $option['tablet']['to'] )   ? $option['tablet']['to'] : '1200';

            $mobile_from   = isset( $option['mobile']['from'] )   ? $option['mobile']['from'] : '250';
            $mobile_to   = isset( $option['mobile']['to'] )   ? $option['mobile']['to'] : '767';

            $hide = isset( $option['hide'] ) ? $option['hide'] : '249';

            $action_hide_menu = isset( $option['action_hide_menu'] ) ? $option['action_hide_menu'] : 'mouseleave';
            $tablet_display = isset( $option['tablet_display'] ) ? $option['tablet_display'] : 'desktop';

            $action_hide_options = array(
                'mouseleave' => __( 'Mouse leaves the menu', 'ova-megamenu' ),
                'click'  => __( 'Mouse click outside the menu', 'ova-megamenu' ),
            );

            $tablet_display_options = array(
                'desktop' => __( 'Desktop', 'ova-megamenu' ),
                'mobile'  => __( 'Mobile', 'ova-megamenu' ),
            );

            $html = '<ul class="ova-panels">';

                /* Desktop */

                $html .= '<li class="panel-item show_desktop_menu">';

                $html .= '<h5 class="title">'.esc_html__( 'Show Desktop Menu', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="panel-table">';

                        $html .= '<li class="panel-row">';

                            $html .= '<ul class="ova-list">';

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Max Width(px)', 'ova-megamenu' ), '[desktop][max_width]', $desktop_max_width );

                            $html .= '</ul>';

                        $html .= '</li>';

                        $html .= '<li class="panel-row">';

                            $html .= '<ul class="ova-list">';

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Min Height(px)', 'ova-megamenu' ), '[desktop][min_height]', $desktop_min_height );

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max Height(px)', 'ova-megamenu' ), '[desktop][max_height]', $desktop_max_height );

                            $html .= '</ul>';

                        $html .= '</li>';

                        $html .= '<li class="panel-row">';

                            $html .= '<h4 class="subtitle">'.esc_html__( 'Action to hide menu', 'ova-megamenu' ).'</h4>';

                            $html .= '<ul class="ova-list inline-flex">';

                            $html .= $this->callback_ova_item_radio( $args,'[action_hide_menu]', $action_hide_menu ,$action_hide_options );

                            $html .= '</ul>';

                        $html .= '</li>';

                    $html .= '</ul>';

                $html .= '</li>';

                /* Tablet */

                $html .= '<li class="panel-item show_tablet_menu">';

                    $html .= '<h5 class="title">'.esc_html__( 'Show Tablet Menu', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="panel-table">';

                        $html .= '<li class="panel-row">';

                            $html .= '<ul class="ova-list">';

                                /* Width from */

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Screen From', 'ova-megamenu' ), '[tablet][from]', $tablet_from );

                                /* Width to */

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'To
                                    ', 'ova-megamenu' ), '[tablet][to]', $tablet_to );

                                $html .= '<li class="ova-item"><label>px</label></li>';

                            $html .= '</ul>';

                        $html .= '</li>';

                        $html .= '<li class="panel-row">';

                            $html .= '<ul class="ova-list">';

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Max Width(px)', 'ova-megamenu' ), '[tablet][max_width]', $tablet_max_width );

                            $html .= '</ul>';

                        $html .= '</li>';

                        $html .= '<li class="panel-row">';

                            $html .= '<ul class="ova-list">';

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Min Height(px)', 'ova-megamenu' ), '[tablet][min_height]', $tablet_min_height );

                                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max Height(px)', 'ova-megamenu' ), '[tablet][max_height]', $tablet_max_height );

                            $html .= '</ul>';

                        $html .= '</li>';

                        $html .= '<li class="panel-row">';

                            $html .= '<h4 class="subtitle">'.esc_html__( 'Style Menu in Tablet like', 'ova-megamenu' ).'</h4>';

                            $html .= '<ul class="ova-list inline-flex">';

                                $html .= $this->callback_ova_item_radio( $args,'[tablet_display]', $tablet_display ,$tablet_display_options );

                            $html .= '</ul>';

                        $html .= '</li>';

                    $html .= '</ul>';
                    

                    $html .= '<p class="description">'.esc_html__( 'Display tablet menu within a specified screen width range.', 'ova-megamenu' ).'</p>';

                $html .= '</li>';

                /* Mobile */

                $html .= '<li class="panel-item show_mobile_menu">';

                    $html .= '<h5 class="title">'.esc_html__( 'Show Mobile Menu', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="panel-table">';

                            $html .= '<li class="panel-row">';

                                $html .= '<ul class="ova-list">';

                                    /* Width from */

                                    $html .= $this->callback_ova_item_number( $args, esc_html__( 'Screen From', 'ova-megamenu' ), '[mobile][from]', $mobile_from );

                                    /* Width to */

                                    $html .= $this->callback_ova_item_number( $args, esc_html__( 'To
                                        ', 'ova-megamenu' ), '[mobile][to]', $mobile_to );

                                    $html .= '<li class="ova-item"><label>px</label></li>';

                                $html .= '</ul>';

                            $html .= '</li>';

                            $html .= '<li class="panel-row">';

                                $html .= '<ul class="ova-list">';

                                    $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Max Width(px)', 'ova-megamenu' ), '[mobile][max_width]', $mobile_max_width );

                                $html .= '</ul>';

                            $html .= '</li>';

                            $html .= '<li class="panel-row">';

                                $html .= '<ul class="ova-list">';

                                    $html .= $this->callback_ova_item_number( $args, esc_html__( 'Menu: Min Height(px)', 'ova-megamenu' ), '[mobile][min_height]', $mobile_min_height );

                                    $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max height(px)', 'ova-megamenu' ), '[mobile][max_height]', $mobile_max_height );

                                $html .= '</ul>';

                            $html .= '</li>';

                        $html .= '</ul>';

                        $html .= '<p class="description">'.esc_html__( 'Display mobile menu within a specified screen width range.', 'ova-megamenu' ).'</p>';

                $html .= '</li>';

                /* Hide Menu */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Hide Menu', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max Width(px)', 'ova-megamenu' ), '[hide]', $hide );

                    $html .= '</ul>';

                    $html .= '<p class="description">'.esc_html__( 'Hide menu from a specified screen width.', 'ova-megamenu' ).'</p>';

                $html .= '</li>';

            $html .= '</ul>';

            $html  .= $this->get_field_description( $args );

            echo $html;
        }

        function callback_column_style( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $background_direction   = isset( $option['background']['direction'] )   ? $option['background']['direction'] : '';
            $background_from        = isset( $option['background']['from'] )        ? $option['background']['from'] : '';
            $background_to          = isset( $option['background']['to'] )          ? $option['background']['to'] : '';

            $padding_left   = isset( $option['padding']['left'] )   ? $option['padding']['left'] : '';
            $padding_top    = isset( $option['padding']['top'] )    ? $option['padding']['top'] : '';
            $padding_right  = isset( $option['padding']['right'] )  ? $option['padding']['right'] : '';
            $padding_bottom = isset( $option['padding']['bottom'] ) ? $option['padding']['bottom'] : '';

            $sperator_enable    = isset( $option['sperator']['enable'] )    ? $option['sperator']['enable'] : '';
            $sperator_width     = isset( $option['sperator']['width'] )     ? $option['sperator']['width'] : '';
            $sperator_color     = isset( $option['sperator']['color'] )     ? $option['sperator']['color'] : '';
            $sperator_position  = isset( $option['sperator']['position'] )  ? $option['sperator']['position'] : 'right';

            $direction          = isset( $option['flex_box']['direction'] )         ? $option['flex_box']['direction'] : '';
            $justify_content    = isset( $option['flex_box']['justify_content'] )   ? $option['flex_box']['justify_content'] : '';
            $justify_items      = isset( $option['flex_box']['justify_items'] )     ? $option['flex_box']['justify_items'] : '';
            $align_content      = isset( $option['flex_box']['align_content'] )     ? $option['flex_box']['align_content'] : '';
            $align_items        = isset( $option['flex_box']['align_items'] )       ? $option['flex_box']['align_items'] : '';

            $margin_left    = isset( $option['margin']['left'] )    ? $option['margin']['left'] : '';
            $margin_top     = isset( $option['margin']['top'] )     ? $option['margin']['top'] : '';
            $margin_right   = isset( $option['margin']['right'] )   ? $option['margin']['right'] : '';
            $margin_bottom  = isset( $option['margin']['bottom'] )  ? $option['margin']['bottom'] : '';

            $border_top     = isset( $option['border']['top'] )     ? $option['border']['top'] : '';
            $border_right   = isset( $option['border']['right'] )   ? $option['border']['right'] : '';
            $border_bottom  = isset( $option['border']['bottom'] )  ? $option['border']['bottom'] : '';
            $border_left    = isset( $option['border']['left'] )    ? $option['border']['left'] : '';
            $border_style   = isset( $option['border']['style'] )   ? $option['border']['style'] : '';
            $border_color   = isset( $option['border']['color'] )   ? $option['border']['color'] : '';

            $border_radius_top_left     = isset( $option['border_radius']['top-left'] )     ? $option['border_radius']['top-left'] : '';
            $border_radius_top_right    = isset( $option['border_radius']['top-right'] )    ? $option['border_radius']['top-right'] : '';
            $border_radius_bottom_right = isset( $option['border_radius']['bottom-right'] ) ? $option['border_radius']['bottom-right'] : '';
            $border_radius_bottom_left  = isset( $option['border_radius']['bottom-left'] )  ? $option['border_radius']['bottom-left'] : '';

            $box_shadow_color       = isset( $option['box_shadow']['color'] )       ? $option['box_shadow']['color'] : '';
            $box_shadow_vertical    = isset( $option['box_shadow']['vertical'] )    ? $option['box_shadow']['vertical'] : '';
            $box_shadow_horizontal  = isset( $option['box_shadow']['horizontal'] )  ? $option['box_shadow']['horizontal'] : '';
            $box_shadow_blur        = isset( $option['box_shadow']['blur'] )        ? $option['box_shadow']['blur'] : '';
            $box_shadow_spread      = isset( $option['box_shadow']['spread'] )      ? $option['box_shadow']['spread'] : '';

            $border_style_opts = $this->border_style;

            $sperator_position_opts = array(
                'right' => esc_html__( 'Right', 'ova-megamenu' ),
                'left' => esc_html__( 'Left', 'ova-megamenu' ),
            );

            $direction_arr = array(
                'row'               => esc_html__( 'Row', 'ova-megamenu' ),
                'row-reverse'       => esc_html__( 'Row Reverse', 'ova-megamenu' ),
                'row-reverse'       => esc_html__( 'Row Reverse', 'ova-megamenu' ),
                'column'            => esc_html__( 'Column', 'ova-megamenu' ),
                'column-reverse'    => esc_html__( 'Column Reverse', 'ova-megamenu' ),
            );

            $justify_content_arr = array(
                'flex-start'    => esc_html__( 'Flex Start', 'ova-megamenu' ),
                'flex-end'      => esc_html__( 'Flex End', 'ova-megamenu' ),
                'center'        => esc_html__( 'Center', 'ova-megamenu' ),
                'space-between' => esc_html__( 'Space Between', 'ova-megamenu' ),
                'space-around'  => esc_html__( 'Space Around', 'ova-megamenu' ),
                'space-evenly'  => esc_html__( 'Space Evenly', 'ova-megamenu' ),
            );

            $justify_items_arr = array(
                'stretch'   => esc_html__( 'Stretch', 'ova-megamenu' ),
                'center'    => esc_html__( 'Center', 'ova-megamenu' ),
                'flex-start'     => esc_html__( 'Flex Start', 'ova-megamenu' ),
                'flex-end'       => esc_html__( 'Flex End', 'ova-megamenu' ),
            );

            $align_content_arr = array(
                'stretch'       => esc_html__( 'Stretch', 'ova-megamenu' ),
                'center'        => esc_html__( 'Center', 'ova-megamenu' ),
                'flex-start'    => esc_html__( 'Flex Start', 'ova-megamenu' ),
                'flex-end'      => esc_html__( 'Flex End', 'ova-megamenu' ),
                'space-between' => esc_html__( 'Space Between', 'ova-megamenu' ),
                'space-around'  => esc_html__( 'Space Around', 'ova-megamenu' ),
                'space-evenly'  => esc_html__( 'Space Evenly', 'ova-megamenu' ),
            );

            $align_items_arr = array(
                'stretch'       => esc_html__( 'Stretch', 'ova-megamenu' ),
                'center'        => esc_html__( 'Center', 'ova-megamenu' ),
                'flex-start'    => esc_html__( 'Flex Start', 'ova-megamenu' ),
                'flex-end'      => esc_html__( 'Flex End', 'ova-megamenu' ),
                'baseline'      => esc_html__( 'Baseline', 'ova-megamenu' ),
            );

            $html = '<ul class="ova-panels">';

                /* padding */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Padding', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[padding][left]', $padding_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[padding][top]', $padding_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[padding][right]', $padding_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[padding][bottom]', $padding_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* margin */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Margin', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[margin][left]', $margin_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[margin][top]', $margin_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[margin][right]', $margin_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[margin][bottom]', $margin_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="ova-list">';

                            $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background][direction]', $background_direction );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background][from]', $background_from );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background][to]', $background_to );

                        $html .= '</ul>';

                $html .= '</li>';

                /* border */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[border][top]', $border_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[border][right]', $border_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[border][bottom]', $border_bottom );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[border][left]', $border_left );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[border][style]', $border_style, $border_style_opts );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[border][color]', $border_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border radius */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border Radius', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[border_radius][top-left]', $border_radius_top_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[border_radius][top-right]', $border_radius_top_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[border_radius][bottom-right]', $border_radius_bottom_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[border_radius][bottom-left]', $border_radius_bottom_left );

                    $html .= '</ul>';

                $html .= '</li>';

                /* box shadow */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Box Shadow', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[box_shadow][horizontal]', $box_shadow_horizontal );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[box_shadow][vertical]', $box_shadow_vertical );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[box_shadow][blur]', $box_shadow_blur );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[box_shadow][spread]', $box_shadow_spread );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[box_shadow][color]', $box_shadow_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* sperator */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Sperator', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_checkbox( $args, esc_html__( 'Enable', 'ova-megamenu' ), '[sperator][enable]', $sperator_enable );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Width(px)', 'ova-megamenu' ), '[sperator][width]', $sperator_width );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[sperator][color]', $sperator_color );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Position', 'ova-megamenu' ), '[sperator][position]', $sperator_position, $sperator_position_opts );

                    $html .= '</ul>';

                $html .= '</li>';

                /* Flex box */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Flexbox', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        /* Direction */

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[flex_box][direction]', $direction, $direction_arr );

                        /* Justify content */

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Justify Content', 'ova-megamenu' ), '[flex_box][justify_content]', $justify_content, $justify_content_arr );

                        /* Justify items */

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Justify Items', 'ova-megamenu' ), '[flex_box][justify_items]', $justify_items, $justify_items_arr );

                        /* Align content */

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Align Content', 'ova-megamenu' ), '[flex_box][align_content]', $align_content, $align_content_arr );


                        /* Align items */

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Align Items', 'ova-megamenu' ), '[flex_box][align_items]', $align_items, $align_items_arr );

                    $html .= '</ul>';

                $html .= '</li>';

            $html .= '</ul>';

            echo $html;
        }

        function callback_row_style( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $background_direction   = isset( $option['background']['direction'] )   ? $option['background']['direction'] : '';
            $background_from        = isset( $option['background']['from'] )        ? $option['background']['from'] : '';
            $background_to          = isset( $option['background']['to'] )          ? $option['background']['to'] : '';

            $padding_left   = isset( $option['padding']['left'] )   ? $option['padding']['left'] : '';
            $padding_top    = isset( $option['padding']['top'] )    ? $option['padding']['top'] : '';
            $padding_right  = isset( $option['padding']['right'] )  ? $option['padding']['right'] : '';
            $padding_bottom = isset( $option['padding']['bottom'] ) ? $option['padding']['bottom'] : '';

            $margin_left    = isset( $option['margin']['left'] )    ? $option['margin']['left'] : '';
            $margin_top     = isset( $option['margin']['top'] )     ? $option['margin']['top'] : '';
            $margin_right   = isset( $option['margin']['right'] )   ? $option['margin']['right'] : '';
            $margin_bottom  = isset( $option['margin']['bottom'] )  ? $option['margin']['bottom'] : '';

            $border_top     = isset( $option['border']['top'] )     ? $option['border']['top'] : '';
            $border_right   = isset( $option['border']['right'] )   ? $option['border']['right'] : '';
            $border_bottom  = isset( $option['border']['bottom'] )  ? $option['border']['bottom'] : '';
            $border_left    = isset( $option['border']['left'] )    ? $option['border']['left'] : '';
            $border_style   = isset( $option['border']['style'] )   ? $option['border']['style'] : '';
            $border_color   = isset( $option['border']['color'] )   ? $option['border']['color'] : '';

            $border_radius_top_left     = isset( $option['border_radius']['top-left'] )     ? $option['border_radius']['top-left'] : '';
            $border_radius_top_right    = isset( $option['border_radius']['top-right'] )    ? $option['border_radius']['top-right'] : '';
            $border_radius_bottom_right = isset( $option['border_radius']['bottom-right'] ) ? $option['border_radius']['bottom-right'] : '';
            $border_radius_bottom_left  = isset( $option['border_radius']['bottom-left'] )  ? $option['border_radius']['bottom-left'] : '';

            $box_shadow_color       = isset( $option['box_shadow']['color'] )       ? $option['box_shadow']['color'] : '';
            $box_shadow_vertical    = isset( $option['box_shadow']['vertical'] )    ? $option['box_shadow']['vertical'] : '';
            $box_shadow_horizontal  = isset( $option['box_shadow']['horizontal'] )  ? $option['box_shadow']['horizontal'] : '';
            $box_shadow_blur        = isset( $option['box_shadow']['blur'] )        ? $option['box_shadow']['blur'] : '';
            $box_shadow_spread      = isset( $option['box_shadow']['spread'] )      ? $option['box_shadow']['spread'] : '';

            $border_style_opts = $this->border_style;

            $html = '<ul class="ova-panels">';

                /* padding */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Padding', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[padding][left]', $padding_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[padding][top]', $padding_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[padding][right]', $padding_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[padding][bottom]', $padding_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* margin */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Margin', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[margin][left]', $margin_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[margin][top]', $margin_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[margin][right]', $margin_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[margin][bottom]', $margin_bottom );

                    $html .= '</ul>';

                $html .= '</li>';

                /* background */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Background', 'ova-megamenu' ).'</h5>';

                        $html .= '<ul class="ova-list">';

                            $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[background][direction]', $background_direction );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[background][from]', $background_from );

                            $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[background][to]', $background_to );

                        $html .= '</ul>';

                $html .= '</li>';

                /* border */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top(px)', 'ova-megamenu' ), '[border][top]', $border_top );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Right(px)', 'ova-megamenu' ), '[border][right]', $border_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom(px)', 'ova-megamenu' ), '[border][bottom]', $border_bottom );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Left(px)', 'ova-megamenu' ), '[border][left]', $border_left );

                        $html .= $this->callback_ova_item_select( $args, esc_html__( 'Style', 'ova-megamenu' ), '[border][style]', $border_style, $border_style_opts );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[border][color]', $border_color );

                    $html .= '</ul>';

                $html .= '</li>';

                /* border radius */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Border Radius', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Left(px)', 'ova-megamenu' ), '[border_radius][top-left]', $border_radius_top_left );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Top Right(px)', 'ova-megamenu' ), '[border_radius][top-right]', $border_radius_top_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Right(px)', 'ova-megamenu' ), '[border_radius][bottom-right]', $border_radius_bottom_right );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Bottom Left(px)', 'ova-megamenu' ), '[border_radius][bottom-left]', $border_radius_bottom_left );

                    $html .= '</ul>';

                $html .= '</li>';

                /* box shadow */

                $html .= '<li class="panel-item">';

                    $html .= '<h5 class="title">'.esc_html__( 'Box Shadow', 'ova-megamenu' ).'</h5>';

                    $html .= '<ul class="ova-list">';

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Horizontal(px)', 'ova-megamenu' ), '[box_shadow][horizontal]', $box_shadow_horizontal );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Vertical(px)', 'ova-megamenu' ), '[box_shadow][vertical]', $box_shadow_vertical );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Blur(px)', 'ova-megamenu' ), '[box_shadow][blur]', $box_shadow_blur );

                        $html .= $this->callback_ova_item_number( $args, esc_html__( 'Spread(px)', 'ova-megamenu' ), '[box_shadow][spread]', $box_shadow_spread );

                        $html .= $this->callback_ova_item_color( $args, esc_html__( 'Color', 'ova-megamenu' ), '[box_shadow][color]', $box_shadow_color );

                    $html .= '</ul>';

                $html .= '</li>';

            $html .= '</ul>';

            echo $html;
        }

        /**
         * Displays a number field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_number( $args ) {
            $value       = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size        = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'small';
            $type        = isset( $args['type'] ) ? $args['type'] : 'number';
            $placeholder = empty( $args['placeholder'] ) ? '' : ' placeholder="' . $args['placeholder'] . '"';
            $min         = ( $args['min'] == '' ) ? '' : ' min="' . $args['min'] . '"';
            $max         = ( $args['max'] == '' ) ? '' : ' max="' . $args['max'] . '"';
            $step        = ( $args['step'] == '' ) ? '' : ' step="' . $args['step'] . '"';

            $html        = sprintf( '<input type="%1$s" class="%2$s-text" id="%3$s[%4$s][%5$s]" name="%3$s[%4$s][%5$s]" value="%6$s"%7$s%8$s%9$s%10$s/>', $type, $size, $args['page_id'] ,$args['section'], $args['id'], $value, $placeholder, $min, $max, $step );
            $html       .= $this->get_field_description( $args );

            echo $html;
        }

        /**
         * Displays a checkbox for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_checkbox( $args ) {

            $value = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );

            $html  = '<fieldset>';
            $html  .= sprintf( '<label for="wpuf-%1$s[%2$s][%3$s]">', $args['page_id'] ,$args['section'], $args['id'] );
            $html  .= sprintf( '<input type="hidden" name="%1$s[%2$s][%3$s]" value="off" />', $args['page_id'] ,$args['section'], $args['id'] );
            $html  .= sprintf( '<input type="checkbox" class="checkbox" id="wpuf-%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="on" %4$s />', $args['page_id'] ,$args['section'], $args['id'], checked( $value, 'on', false ) );
            $html  .= sprintf( '%1$s</label>', $args['desc'] );
            $html  .= '</fieldset>';

            echo $html;
        }

        /**
         * Displays a multicheckbox for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_multicheck( $args ) {

            $value = $this->get_option( $args['id'], $args['section'], $args['std'] );
            $html  = '<fieldset>';
            $html .= sprintf( '<input type="hidden" name="%1$s[%2$s][%3$s]" value="" />', $args['page_id'] ,$args['section'], $args['id'] );
            foreach ( $args['options'] as $key => $label ) {
                $checked = isset( $value[$key] ) ? $value[$key] : '0';
                $html    .= sprintf( '<label for="wpuf-%1$s[%2$s][%3$s][%4$s]">',$args['page_id'] ,$args['section'], $args['id'], $key );
                $html    .= sprintf( '<input type="checkbox" class="checkbox" id="wpuf-%1$s[%2$s][%3$s][%4$s]" name="%1$s[%2$s][%3$s][%4$s]" value="%4$s" %5$s />',$args['page_id'] ,$args['section'], $args['id'], $key, checked( $checked, $key, false ) );
                $html    .= sprintf( '%1$s</label><br>',  $label );
            }

            $html .= $this->get_field_description( $args );
            $html .= '</fieldset>';

            echo $html;
        }

        /**
         * Displays a radio button for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_radio( $args ) {

            $value = $this->get_option( $args['id'], $args['section'], $args['std'] );
            $html  = '<ul class="ova-list">';

            foreach ( $args['options'] as $key => $label ) {
                $html .= '<li class="ova-item">';
                    $html .= sprintf( '<label for="wpuf-%1$s[%2$s][%3$s][%4$s]">', $args['page_id'] , $args['section'], $args['id'], $key );
                    $html .= sprintf( '<input type="radio" class="radio" id="wpuf-%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="%4$s" %5$s />', $args['page_id'] ,$args['section'], $args['id'], $key, checked( $value, $key, false ) );
                    $html .= sprintf( '%1$s</label><br>', $label );
                $html .= '</li>';
            }

            
            $html .= '</ul>';

            $html .= $this->get_field_description( $args );

            echo $html;
        }

        function callback_ova_item_radio( $args, $key, $value, $options ){

            $html = '';

            foreach ( $options as $_key => $label ) {
                $html .= '<li class="ova-item">';
                    $html .= sprintf( '<label for="wpuf-%1$s[%2$s][%3$s]%4$s">', $args['page_id'] , $args['section'], $args['id'], $key );
                    $html .= sprintf( '<input type="radio" class="radio" id="wpuf-%1$s[%2$s][%3$s]%4$s" name="%1$s[%2$s][%3$s]%4$s" value="%5$s" %6$s />', $args['page_id'] ,$args['section'], $args['id'], $key, $_key, checked( $value, $_key, false ) );
                    $html .= sprintf( '%1$s</label><br>', $label );
                $html .= '</li>';
            }

            return $html;
        }

        /**
         * Displays a selectbox for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_select( $args ) {

            $value = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size  = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';
            $html  = sprintf( '<select class="%1$s" name="%2$s[%3$s][%4$s]" id="%2$s[%3$s][%4$s]">', $size, $args['page_id'] ,$args['section'], $args['id'] );

            foreach ( $args['options'] as $key => $label ) {
                $html .= sprintf( '<option value="%s"%s>%s</option>', $key, selected( $value, $key, false ), $label );
            }

            $html .= sprintf( '</select>' );
            $html .= $this->get_field_description( $args );

            echo $html;
        }

        /**
         * Displays a textarea for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_textarea( $args ) {

            $value       = esc_textarea( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size        = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';
            $placeholder = empty( $args['placeholder'] ) ? '' : ' placeholder="'.$args['placeholder'].'"';

            $html        = sprintf( '<textarea rows="5" cols="55" class="%1$s-text" id="%2$s[%3$s][%4$s]" name="%2$s[%3$s][%4$s]"%5$s>%6$s</textarea>', $size, $args['page_id'] ,$args['section'], $args['id'], $placeholder, $value );
            $html        .= $this->get_field_description( $args );

            echo $html;
        }

        function callback_code_editor( $args ){

            $value       = esc_textarea( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size        = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';
            $placeholder = empty( $args['placeholder'] ) ? '' : ' placeholder="'.$args['placeholder'].'"';

            $html        = sprintf( '<textarea rows="5" cols="55" id="code_editor" class="%1$s-text" name="%2$s[%3$s][%4$s]"%5$s>%6$s</textarea>', $size, $args['page_id'], $args['section'], $args['id'], $placeholder, $value );
            $html        .= $this->get_field_description( $args );

            echo $html;
        }

        function callback_css_output( $args ){
            $html = '';
            echo $html;
        }

        /**
         * Displays the html for a settings field
         *
         * @param array   $args settings field args
         * @return string
         */
        function callback_html( $args ) {
            echo $this->get_field_description( $args );
        }

        /**
         * Displays a rich text textarea for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_wysiwyg( $args ) {

            $value = $this->get_option( $args['id'], $args['section'], $args['std'] );
            $size  = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : '500px';

            echo '<div style="max-width: ' . $size . ';">';

            $editor_settings = array(
                'teeny'         => true,
                'textarea_name' => $args['page_id'].'['.$args['section'].']' . '[' . $args['id'] . ']',
                'textarea_rows' => 10
            );

            if ( isset( $args['options'] ) && is_array( $args['options'] ) ) {
                $editor_settings = array_merge( $editor_settings, $args['options'] );
            }

            wp_editor( $value, $args['page_id']. '-' .$args['section'] . '-' . $args['id'], $editor_settings );

            echo '</div>';

            echo $this->get_field_description( $args );
        }

        /**
         * Displays a file upload field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_file( $args ) {

            $value = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size  = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';
            $id    = $args['page_id'].'['.$args['section']. ']' . '[' . $args['id'] . ']';
            $label = isset( $args['options']['button_label'] ) ? $args['options']['button_label'] : __( 'Choose File' );

            $html  = sprintf( '<input type="text" class="%1$s-text wpsa-url" id="%2$s[%3$s][%4$s]" name="%2$s[%3$s][%4$s]" value="%5$s"/>', $size, $args['page_id'], $args['section'], $args['id'], $value );
            $html  .= '<input type="button" class="button wpsa-browse" value="' . $label . '" />';
            $html  .= $this->get_field_description( $args );

            echo $html;
        }

        /**
         * Displays a password field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_password( $args ) {

            $value = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size  = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';

            $html  = sprintf( '<input type="password" class="%1$s-text" id="%2$s[%3$s][%4$s]" name="%2$s[%3$s][%4$s]" value="%5$s"/>', $size, $args['page_id'], $args['section'], $args['id'], $value );
            $html  .= $this->get_field_description( $args );

            echo $html;
        }

        /**
         * Displays a color picker field for a settings field
         *
         * @param array   $args settings field args
         */
        function callback_color( $args ) {

            $value = esc_attr( $this->get_option( $args['id'], $args['section'], $args['std'] ) );
            $size  = isset( $args['size'] ) && !is_null( $args['size'] ) ? $args['size'] : 'regular';

            $html  = sprintf( '<input type="text" class="%1$s-text wp-color-picker-field" id="%2$s[%3$s][%4$s]" name="%2$s[%3$s][%4$s]" value="%5$s" data-default-color="%6$s" />', $size, $args['page_id'] ,$args['section'], $args['id'], $value, $args['std'] );
            $html  .= $this->get_field_description( $args );

            echo $html;
        }

        function callback_background_color( $args ) {

            $option = $this->get_option( $args['id'], $args['section'], array() );

            $background_direction   = isset( $option['direction'] ) ? $option['direction'] : '';
            $background_from        = isset( $option['from'] )      ? $option['from'] : '#fff';
            $background_to          = isset( $option['to'] )        ? $option['to'] : '#fff';

            $html = '<ul class="ova-list">';

                $html .= $this->callback_ova_item_select_background_direction( $args, esc_html__( 'Direction', 'ova-megamenu' ), '[direction]', $background_direction );

                $html .= $this->callback_ova_item_color( $args, esc_html__( 'From', 'ova-megamenu' ), '[from]', $background_from );

                $html .= $this->callback_ova_item_color( $args, esc_html__( 'To', 'ova-megamenu' ), '[to]', $background_to );

            $html .= '</ul>';

            echo $html;
        }

        function callback_width_height( $args ){
            $option = $this->get_option( $args['id'], $args['section'], array() );

            $min_width  = isset( $option['min_width'] )     ? $option['min_width'] : '';
            $max_width  = isset( $option['max_width'] )     ? $option['max_width'] : '';
            $min_height = isset( $option['min_height'] )    ? $option['min_height'] : '';
            $max_height = isset( $option['max_height'] )    ? $option['max_height'] : '';
            $html = '';

            $html .= '<ul class="ova-list">';

                /* Min width */

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Min width(px)', 'ova-megamenu' ), '[min_width]', $min_width );

                /* Max width */

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max width(px)', 'ova-megamenu' ), '[max_width]', $max_width );

                /* Min height */

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Min height(px)', 'ova-megamenu' ), '[min_height]', $min_height );

                /* Max height */

                $html .= $this->callback_ova_item_number( $args, esc_html__( 'Max height(px)', 'ova-megamenu' ), '[max_height]', $max_height );

            $html .= '</ul>';

            echo $html;
        }


        /**
         * Displays a select box for creating the pages select box
         *
         * @param array   $args settings field args
         */
        function callback_pages( $args ) {

            $dropdown_args = array(
                'selected' => esc_attr($this->get_option($args['id'], $args['section'], $args['std'] ) ),
                'name'     => $args['page_id'].'['. $args['section'] . ']' . '[' . $args['id'] . ']',
                'id'       => $args['page_id'].'['. $args['section'] . ']' . '[' . $args['id'] . ']',
                'echo'     => 0
            );
            $html = wp_dropdown_pages( $dropdown_args );
            echo $html;
        }

        /**
         * Sanitize callback for Settings API
         *
         * @return mixed
         */
        function sanitize_options( $options ) {

            if ( !$options ) {
                return $options;
            }

            foreach( $options as $option_slug => $option_value ) {
                $sanitize_callback = $this->get_sanitize_callback( $option_slug );

                // If callback is set, call it
                if ( $sanitize_callback ) {
                    $options[ $option_slug ] = call_user_func( $sanitize_callback, $option_value );
                    continue;
                }
            }

            return $options;
        }

        /**
         * Get sanitization callback for given option slug
         *
         * @param string $slug option slug
         *
         * @return mixed string or bool false
         */
        function get_sanitize_callback( $slug = '' ) {

            if ( empty( $slug ) ) {
                return false;
            }

            // Iterate over registered fields and see if we can find proper callback
            foreach( $this->get_settings_fields() as $section => $options ) {
                foreach ( $options as $option ) {
                    if ( $section != $slug ) {
                        continue;
                    }

                    // Return the callback name
                    return isset( $option['sanitize_callback'] ) && is_callable( $option['sanitize_callback'] ) ? $option['sanitize_callback'] : false;
                }
            }

            return false;
        }

        /**
         * Get the value of a settings field
         *
         * @param string  $option  settings field name
         * @param string  $section the section name this field belongs to
         * @param string  $default default text if it's not found
         * @return string
         */
        function get_option( $option, $section, $default = '' ) {
            $page_id = $this->page_id;
            $options = get_option( $page_id );

            if ( isset( $options[$section][$option] ) ) {
                return $options[$section][$option];
            }

            return $default;
        }

    }

    new OVAM_Settings();
}
