<?php

if ( ! defined('ABSPATH') ) {
	exit();
}

if ( ! class_exists('OVAM_Tools') ) {
	

	class OVAM_Tools {

		protected $sections;

		public function __construct(){
			$this->sections = $this->get_settings_sections();
			add_action( 'admin_menu', array($this, 'admin_menu') );
		}

		function admin_menu(){
			add_submenu_page( 'edit.php?post_type=ova-megamenu', esc_html__( 'Tools', 'ova-megamenu' ), esc_html__( 'Tools', 'ova-megamenu' ), 'manage_options', 'ova_megamenu_tools', array($this, 'plugin_page') );
		}

		function get_settings_sections(){
			return array(
				array(
						'id' => 'general',
						'title' => esc_html__( 'General', 'ova-megamenu' )
					),
					array(
						'id' => 'import_export',
						'title' => esc_html__( 'Import/Export', 'ova-megamenu' )
					)
				);
		}

		function get_import_export_template(){
			?>
			<table class="form-table" role="presentation">
				<tbody>
					<tr>
						<th scope="row"><label for=""><?php esc_html_e( 'Export Settings', 'ova-megamenu' ); ?></label></th>
						<td>
							<button type="button"
							data-nonce="<?php echo esc_attr( wp_create_nonce( 'ova_export_settings' ) ); ?>"
							data-title="<?php esc_attr_e( 'Export Settings', 'ova-megamenu' ); ?>"
							class="ova_export_settings_popup btn_icon_text button button-secondary">
								<span class="dashicons dashicons-upload"></span>
								<span class="text"><?php esc_html_e( 'Export', 'ova-megamenu' ); ?></span>
							</button>
						</td>
					</tr>
					<tr>
						<th scope="row"><label for=""><?php esc_html_e( 'Import Settings', 'ova-megamenu' ); ?></label></th>
						<td>
							<button type="button"
							data-nonce="<?php echo esc_attr( wp_create_nonce( 'ova_import_settings' ) ); ?>"
							data-title="<?php esc_attr_e( 'Import Settings', 'ova-megamenu' ); ?>"
							data-button="<?php esc_attr_e( 'Import', 'ova-megamenu' ); ?>"
							class="ova_import_settings_popup btn_icon_text button button-secondary">
								<span class="dashicons dashicons-download"></span>
								<span class="text"><?php esc_html_e( 'Import', 'ova-megamenu' ); ?></span>
							</button>
						</td>
					</tr>
					
				</tbody>
			</table>
			<p class="description"><?php esc_html_e( 'Import and export data in mega menu settings, convenient tool for data backup.', 'ova-megamenu' ); ?></p>
			<?php
		}

		function get_general_template(){
			$button_arr = array(
				'yes' => esc_html__( 'Yes', 'ova-megamenu' ),
				'no' => esc_html__( 'No', 'ova-megamenu' ),
			);
			?>
			<table class="form-table" role="presentation">
				<tbody>
					<tr>
						<th scope="row"><label for=""><?php esc_html_e( 'Reset All Settings', 'ova-megamenu' ); ?></label></th>
						<td>
							<button type="button"
							data-nonce="<?php echo esc_attr( wp_create_nonce( 'ova_reset_settings' ) ); ?>"
							data-title="<?php esc_attr_e( 'Reset Settings', 'ova-megamenu' ); ?>"
							data-button="<?php echo esc_attr( json_encode( $button_arr ) ); ?>"
							data-desc="<?php esc_attr_e( 'All your setup data will be restored to their original state. Do you want to do that?', 'ova-megamenu' ); ?>"
							class="ova_reset_settings btn_icon_text button button-secondary">
								<span class="dashicons dashicons-image-rotate"></span>
								<span class="text"><?php esc_html_e( 'Reset', 'ova-megamenu' ); ?></span>
							</button>
						</td>
					</tr>
				</tbody>
			</table>
			<?php
		}

		function plugin_page(){
			$sections = $this->sections;
			?>
			<div class="ova-megamenu-tools">
				<h2 class="nav-tab-wrapper">
					<?php foreach ( $sections as $key => $section ): ?>
                        <a href="#<?php echo esc_attr( $section['id'] ); ?>" data-id="<?php echo esc_attr( $section['id'] ); ?>" class="nav-tab"><?php echo esc_html( $section['title'] ); ?></a>
                    <?php endforeach; ?>
				</h2>

				
				<?php foreach ( $sections as $key => $section ): ?>
					<div class="tabs-content" id="<?php echo esc_attr( $section['id'] ); ?>">
						<?php
							switch ( $section['id'] ) {
								case 'import_export':
									$this->get_import_export_template();
									break;
								case 'general':
									$this->get_general_template();
									break;
								
								default:
									break;
							}
						?>
					</div>
				<?php endforeach; ?>

			</div>
			<?php
		}

	}

	new OVAM_Tools();
}