<?php

if ( !defined( 'ABSPATH' ) ) exit();

if ( ! class_exists('OVAM_Add_Shortcode') ) {
	class OVAM_Add_Shortcode {

		public function __construct(){
			add_shortcode( 'ova-megamenu', array( $this, 'ova_add_shortcode' ) );
			add_shortcode( 'ova-elementor-template', array( $this, 'ova_elementor_template' ) );
		}

		public function ova_add_shortcode( $atts ){
			global $wp;


			$current_url = trailingslashit( home_url( $wp->request ) );
			$attributes = shortcode_atts( array(
				'id' => 0,
			), $atts );

			if ( get_post_type( $attributes['id'] ) !== 'ova-megamenu' || get_post_status( $attributes['id'] ) !== 'publish'  ) {
				return '';
			}
			$ova_meta_settings = get_post_meta( $attributes['id'], 'ova_megamenu', true );
			$options = get_option( 'ova_megamenu', array() );
			$general_opts = isset( $options['general'] ) ? $options['general'] : array();

			$width_style 		= isset( $ova_meta_settings['width_style'] ) ? $ova_meta_settings['width_style'] : 'full-width';
			$action_hide_menu 	= isset( $general_opts['breakpoint']['action_hide_menu'] ) ? $general_opts['breakpoint']['action_hide_menu'] : 'mouseleave';
			$tablet_display 	= isset( $general_opts['breakpoint']['tablet_display'] ) ? $general_opts['breakpoint']['tablet_display'] : 'desktop';
			$position 			= isset( $general_opts['position'] ) ? $general_opts['position'] : 'center';

			$desktop_max_width 	= isset( $general_opts['breakpoint']['desktop']['max_width'] ) ? $general_opts['breakpoint']['desktop']['max_width'] : '1000';
			$tablet_max_width 	= isset( $general_opts['breakpoint']['tablet']['max_width'] ) ? $general_opts['breakpoint']['tablet']['max_width'] : '900';
			$mobile_max_width 	= isset( $general_opts['breakpoint']['mobile']['max_width'] ) ? $general_opts['breakpoint']['mobile']['max_width'] : '767';


			$tablet_from 	= isset( $general_opts['breakpoint']['tablet']['from'] ) ? $general_opts['breakpoint']['tablet']['from'] : '768';
			$tablet_to 		= isset( $general_opts['breakpoint']['tablet']['to'] ) ? $general_opts['breakpoint']['tablet']['to'] : '1200';

			$mobile_from 	= isset( $general_opts['breakpoint']['mobile']['from'] ) ? $general_opts['breakpoint']['mobile']['from'] : '250';
			$mobile_to 		= isset( $general_opts['breakpoint']['mobile']['to'] ) ? $general_opts['breakpoint']['mobile']['to'] : '767';

			$hide_menu 		= isset( $general_opts['breakpoint']['hide'] ) ? $general_opts['breakpoint']['hide'] : '249';

			$menu_id 		= $attributes['id'];
			$the_content 	= ova_megamenu_get_the_content($menu_id);
			
			$menu_arr = array('desktop','tablet','mobile');

			$output = '<div class="ova_megamenu_container"';
			$output .= ' data-desktop-max-width="'.$desktop_max_width.'"';
			$output .= ' data-tablet-max-width="'.$tablet_max_width.'"';
			$output .= ' data-mobile-max-width="'.$mobile_max_width.'"';
			$output .= ' data-tablet-from="'.$tablet_from.'"';
			$output .= ' data-tablet-to="'.$tablet_to.'"';
			$output .= ' data-mobile-from="'.$mobile_from.'"';
			$output .= ' data-mobile-to="'.$mobile_to.'"';
			$output .= ' data-action-hide="'.$action_hide_menu.'"';
			$output .= ' data-hide-menu="'.$hide_menu.'"';
			$output .= ' data-tablet-display="'.$tablet_display.'"';
			$output .= ' data-width-style="'. esc_attr( $width_style ) .'" data-position="'.esc_attr( $position ).'">';
			if ( ! empty( $the_content ) ) {
				$output .= '<div class="ova_the_content">';
					$output .= $the_content;
				$output .= '</div>';
			}
			foreach ( $menu_arr as $menu_name ) {

				$ova_megamenu = get_post_meta( $menu_id, 'ova_megamenu_'.$menu_name, true );
				if ( ! empty( $ova_megamenu ) ) {
					$output .= '<div class="ova_megamenu '.$menu_name.'" data-menu="'.$menu_name.'">';
					
						$output .= '<div class="ova_megamenu_rows">';
							foreach ( $ova_megamenu as $key => $row ) {
								$output .= '<div class="ova_megamenu_row">';
									if ( ! empty( $row ) ) {
										$output .= '<div class="ova_megamenu_columns">';
										foreach ( $row as $key => $column ) {
											
											if ( ! empty( $column ) ) {
												$column_width = isset( $column['column_width'] ) ? $column['column_width'] : '';
												$output .= '<div class="ova_megamenu_column" style="width:'.$column_width.'%" >';

													if ( ! empty( $column['widget'] ) ) {
														$widgets = $column['widget'];
														
														foreach ( $widgets as $widget ) {
															$output .= '<div class="ova-widget">';
																if ( ! empty( $widget['heading'] ) ) {
																	$output .= '<h3 class="ova-heading">'. esc_html( $widget['heading'] ) .'</h3>';
																}
																$output .= '<div class="ova-content">';
																$widget_type = isset( $widget['type'] ) ? $widget['type'] : '';
																switch ( $widget_type ) {
																	
																	case 'text':
																		$widget_text = isset( $widget['text'] ) ? $widget['text'] : '';
																		$output .= $widget_text;
																		break;

																	case 'shortcode':
																		$widget_shortcode = isset( $widget['shortcode'] ) ? $widget['shortcode'] : '';
																		if ( ! has_shortcode( $widget_shortcode, 'ova-megamenu' ) ) {
																			$output .= do_shortcode( $widget_shortcode );
																		}
																	break;

																	case 'menu':
																		$menu_type = isset( $widget['menu_type'] ) ? $widget['menu_type'] : '';
																		if ( $menu_type === 'select-menu' ) {
																			$widget_menu = isset( $widget['menu'] ) ? $widget['menu'] : [];
																			if ( ! empty( $widget_menu ) ) {
																				$menu = wp_get_nav_menu_object( $widget['menu'] );
																				$menu_items = wp_get_nav_menu_items( $menu );

																				if ( $menu_items ) {
																					$output .= '<div class="ova_megamenu_sub_menu">';
																					foreach ( $menu_items as $key => $item ) {
																						if ( $item->menu_item_parent == 0 ) {
																							$class_active = $current_url == $item->url ? ' active' : '';
																							$classes = ! empty( $item->classes ) ? implode(" ", $item->classes ) : '';
																							$target = $item->target;
																							$title = $item->attr_title;
																							$output .= '<div class="ova_megamenu_item'.esc_attr( $class_active.' '.$classes ).'">';
																								$output .= '<a href="'.esc_url( $item->url ).'" '.ova_megamenu_get_attr_menu_item( $target, $title ).'>';
																									$output .= esc_html( $item->title );
																								$output .= '</a>';
																							$output .= '</div>';
																						}
																					}
																					$output .= '</div>';
																				}
																			}
																		} else {
																			$menu_items = isset( $widget['menu_item'] ) ? $widget['menu_item'] : array();
																			if ( ! empty( $menu_items ) ) {
																				$output .= '<div class="ova_megamenu_sub_menu">';
																				foreach ( $menu_items as $item ) {
																					$label = isset( $item['label'] ) ? $item['label'] : '';
																					$title = isset( $item['title'] ) ? $item['title'] : '';
																					$url = isset( $item['url'] ) ? $item['url'] : '';
																					$target = isset( $item['target'] ) ? '_blank' : '';
																					$class = isset( $item['class'] ) ? $item['class'] : '';
																					$class_active = $current_url == $url ? ' active' : '';
																					$output .= '<div class="ova_megamenu_item'.esc_attr( $class_active ).'">';
																								$output .= '<a href="'.esc_url( $url ).'" '.ova_megamenu_get_attr_menu_item( $target, $title ).'>';
																									$output .= esc_html( $label );
																								$output .= '</a>';
																							$output .= '</div>';
																				}
																				$output .= '</div>';
																			}
																		}

																		
																		
																	break;
																	default:
																		break;
																}
																$output .= '</div>';
															$output .= '</div>';
														}
														
													}
												$output .= '</div>';
											}
											
										}
										$output .= '</div>';
									}
									
								$output .= '</div>';
							}
						$output .= '</div>';
					
					$output .= '</div>';
				}
			}
			$output.= '</div>';
			return $output;
		}

		public function ova_elementor_template( $atts ){
			$attributes = shortcode_atts( array(
				'id' => 0,
			), $atts );

			$content_elementor = '';
			$post_ID = $attributes['id'];
			if ( class_exists("\\Elementor\\Plugin") ) {
			    $plugin_elementor = \Elementor\Plugin::instance();
			    $content_elementor = $plugin_elementor->frontend->get_builder_content($post_ID);
			}
			return $content_elementor;
		}
	}

	new OVAM_Add_Shortcode();
}