<?php

if ( ! defined('ABSPATH') ) {
	exit();
}

if ( ! class_exists('OVAM_Css_Print') ) {
	

	class OVAM_Css_Print {

		private static $_instance = null;

		private $custom_style;

		private $css_output;

		private $options;

		private $google_fonts;

		public function __construct(){
			$this->options = get_option('ova_megamenu', array() );
			$this->google_fonts = new OVAM_Google_Fonts();

			$ova_megamenu_advanced = isset( $this->options['advanced'] ) ? $this->options['advanced'] : array();

			$this->custom_style = isset( $ova_megamenu_advanced['custom_style'] ) ? $ova_megamenu_advanced['custom_style'] : '';
			$this->css_output = isset( $ova_megamenu_advanced['css_output'] ) ? $ova_megamenu_advanced['css_output'] : '';
		}

		public static function instance() {
			if ( !empty( self::$_instance ) ) {
				return self::$_instance;
			}
			return self::$_instance = new self();
		}

		public function get_css_setting(){
			return $this->css_output;
		}

		public function print_css(){
			$style 	= '';
			$html 	= '';
			$style .= $this->get_setting_css();
			$style .= $this->get_custom_style();
			$html .= '<style id="ova-megamenu-internal-style">';
			$html .= $style;
			$html .= '</style>';
			echo $html;
		}

		public function get_custom_style(){
			return $this->custom_style;
		}

		public function get_general_style( $ova_megamenu_opts = array() ){
			$general_style 	= isset( $ova_megamenu_opts['general'] ) ? $ova_megamenu_opts['general'] : array();

			$z_index 			= isset( $general_style['z_index'] ) ? $general_style['z_index'] : '999';
			$background_color 	= isset( $general_style['background'] ) ? implode( ', ', array_filter( $general_style['background'] ) ) : '#fff, #fff' ;

			// desktop

			$max_width = isset( $general_style['breakpoint']['desktop']['max_width'] ) ? $general_style['breakpoint']['desktop']['max_width'] : '1000';
			$min_height = isset( $general_style['breakpoint']['desktop']['min_height'] ) ? $general_style['breakpoint']['desktop']['min_height'] : '';
			$max_height = isset( $general_style['breakpoint']['desktop']['max_height'] ) ? $general_style['breakpoint']['desktop']['max_height'] : '';

			// tablet
			$tablet_from = isset( $general_style['breakpoint']['tablet']['from'] ) ? $general_style['breakpoint']['tablet']['from'] : '768';
			$tablet_to = isset( $general_style['breakpoint']['tablet']['to'] ) ? $general_style['breakpoint']['tablet']['to'] : '1200';

			$tablet_max_width = isset( $general_style['breakpoint']['tablet']['max_width'] ) ? $general_style['breakpoint']['tablet']['max_width'] : '900';
			$tablet_min_height = isset( $general_style['breakpoint']['tablet']['min_height'] ) ? $general_style['breakpoint']['tablet']['min_height'] : '';
			$tablet_max_height = isset( $general_style['breakpoint']['tablet']['max_height'] ) ? $general_style['breakpoint']['tablet']['max_height'] : '';

			// mobile
			$mobile_from = isset( $general_style['breakpoint']['mobile']['from'] ) ? $general_style['breakpoint']['mobile']['from'] : '250';
			$mobile_to = isset( $general_style['breakpoint']['mobile']['to'] ) ? $general_style['breakpoint']['mobile']['to'] : '767';

			$mobile_max_width = isset( $general_style['breakpoint']['mobile']['max_width'] ) ? $general_style['breakpoint']['mobile']['max_width'] : '767';
			$mobile_min_height = isset( $general_style['breakpoint']['mobile']['min_height'] ) ? $general_style['breakpoint']['mobile']['min_height'] : '';
			$mobile_max_height = isset( $general_style['breakpoint']['mobile']['max_height'] ) ? $general_style['breakpoint']['mobile']['max_height'] : '';

			// border
			$border_top 	= isset( $general_style['border']['top'] ) ? $general_style['border']['top'] : '';
			$border_right 	= isset( $general_style['border']['right'] ) ? $general_style['border']['right'] : '';
			$border_bottom 	= isset( $general_style['border']['bottom'] ) ? $general_style['border']['bottom'] : '';
			$border_left 	= isset( $general_style['border']['left'] ) ? $general_style['border']['left'] : '';
			$border_color 	= isset( $general_style['border']['color'] ) ? $general_style['border']['color'] : '';
			$border_style 	= isset( $general_style['border']['style'] ) ? $general_style['border']['style'] : '';

			$padding 		= isset( $general_style['padding'] ) ? $general_style['padding'] : '';
			$border_radius 	= isset( $general_style['border_radius'] ) ? $general_style['border_radius'] : '';

			// box shadow
			$box_shadow_horizontal 	= isset( $general_style['box_shadow']['horizontal'] ) ? $general_style['box_shadow']['horizontal'] : '';
			$box_shadow_vertical 	= isset( $general_style['box_shadow']['vertical'] ) ? $general_style['box_shadow']['vertical'] : '';
			$box_shadow_blur 		= isset( $general_style['box_shadow']['blur'] ) ? $general_style['box_shadow']['blur'] : '';
			$box_shadow_spread 		= isset( $general_style['box_shadow']['spread'] ) ? $general_style['box_shadow']['spread'] : '';
			$box_shadow_color 		= isset( $general_style['box_shadow']['color'] ) ? $general_style['box_shadow']['color'] : '';

			$box_shadow = '';

			if ( $box_shadow_horizontal != '' && $box_shadow_vertical != '' && $box_shadow_blur != '' && $box_shadow_spread != '' && $box_shadow_color != '' ) {
				$box_shadow .= $box_shadow_horizontal.'px';
				$box_shadow .= ' '.$box_shadow_vertical.'px';
				$box_shadow .= ' '.$box_shadow_blur.'px';
				$box_shadow .= ' '.$box_shadow_spread.'px';
				$box_shadow .= ' '.$box_shadow_color;
			}
			$style = '';

			// ova_megamenu_container
			$style .= '.ova-megamenu-item .ova_megamenu_container {';

				if ( $background_color != '' ) {
					$style .= 'background-image: linear-gradient('.$background_color.');';
				}

				if ( $max_width != '' ) {
					$style .= 'max-width: '.$max_width.'px;';
				}

				if ( $min_height != '' ) {
					$style .= 'min-height: '.$min_height.'px;';
				}

				if ( $max_height != '' ) {
					$style .= 'max-height: '.$max_height.'px;';
				}

				if ( $border_top != '' ) {
					$style .= 'border-top-width: '.$border_top.'px;';
				}

				if ( $border_right != '' ) {
					$style .= 'border-right-width: '.$border_right.'px;';
				}

				if ( $border_bottom != '' ) {
					$style .= 'border-bottom-width: '.$border_bottom.'px;';
				}

				if ( $border_left != '' ) {
					$style .= 'border-left-width: '.$border_left.'px;';
				}

				if ( $border_color != '' ) {
					$style .= 'border-color: '.$border_color.';';
				}

				if ( $border_style != '' ) {
					$style .= 'border-style: '.$border_style.';';
				}

				if ( ! empty( $padding ) ) {
					foreach ( $padding as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'padding-'.$pos.': '.$val.'px;';
						}
					}
				}

				if ( ! empty( $border_radius ) ) {
					foreach ( $border_radius as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'border-'.$pos.'-radius: '.$val.'px;';
						}
					}
				}

				if ( $box_shadow != '' ) {
					$style .= 'box-shadow: '.$box_shadow.';';
				}

				if ( $z_index != '' ) {
					$style .= 'z-index: '.$z_index.';';
				}
				
			$style .= '}';

			// media query

			if ( $tablet_from != '' && $tablet_to != '' ) {
				$style .= '@media (min-width: '.$tablet_from.'px) and (max-width: '.$tablet_to.'px) {';

					$style .= '.ova-megamenu-item .ova_megamenu_container {';

						if ( $tablet_max_width != '' ) {
							$style .= 'max-width: '.$tablet_max_width.'px;';
						}

						if ( $tablet_min_height != '' ) {
							$style .= 'min-height: '.$tablet_min_height.'px;';
						}

						if ( $tablet_max_height != '' ) {
							$style .= 'max-height: '.$tablet_max_height.'px;';
						}

					$style .= '}';

				$style .= '}';
			}


			if ( $mobile_from != '' && $mobile_to != '' ) {
				$style .= '@media (min-width: '.$mobile_from.'px) and (max-width: '.$mobile_to.'px) {';

					$style .= '.ova-megamenu-item .ova_megamenu_container {';

						if ( $mobile_max_width != '' ) {
							$style .= 'max-width: '.$mobile_max_width.'px;';
						}

						if ( $mobile_min_height != '' ) {
							$style .= 'min-height: '.$mobile_min_height.'px;';
						}

						if ( $mobile_max_height != '' ) {
							$style .= 'max-height: '.$mobile_max_height.'px;';
						}

					$style .= '}';

				$style .= '}';
			}

			return $style;
		}

		public function get_mega_menus_style( $ova_megamenu_opts = array() ){

			$mega_menus = isset( $ova_megamenu_opts['mega_menus'] ) ? $ova_megamenu_opts['mega_menus'] : array();

			$style = '';

			$style .= $this->get_mega_menus_row_style( $mega_menus );

			$style .= $this->get_mega_menus_column_style( $mega_menus );

			$style .= $this->get_mega_menus_widget_style( $mega_menus );

			$style .= $this->get_mega_menus_heading_style( $mega_menus );

			$style .= $this->get_mega_menus_menu_item_style( $mega_menus );

			return $style;
		}

		public function get_mega_menus_row_style( $mega_menus = array() ){
			$rows_padding = isset( $mega_menus['rows']['padding'] ) ? $mega_menus['rows']['padding'] : array();
			$rows_margin = isset( $mega_menus['rows']['margin'] ) ? $mega_menus['rows']['margin'] : array();
			$rows_background = isset( $mega_menus['rows']['background'] ) ? implode( ', ', array_filter( $mega_menus['rows']['background'] ) ) : '' ;

			$rows_border_top 		= isset( $mega_menus['rows']['border']['top'] ) ? $mega_menus['rows']['border']['top'] : '';
			$rows_border_right 	= isset( $mega_menus['rows']['border']['right'] ) ? $mega_menus['rows']['border']['right'] : '';
			$rows_border_bottom 	= isset( $mega_menus['rows']['border']['bottom'] ) ? $mega_menus['rows']['border']['bottom'] : '';
			$rows_border_left 	= isset( $mega_menus['rows']['border']['left'] ) ? $mega_menus['rows']['border']['left'] : '';
			$rows_border_style 	= isset( $mega_menus['rows']['border']['style'] ) ? $mega_menus['rows']['border']['style'] : '';
			$rows_border_color 	= isset( $mega_menus['rows']['border']['color'] ) ? $mega_menus['rows']['border']['color'] : '';

			$rows_border_radius 	= isset( $mega_menus['rows']['border_radius'] ) ? $mega_menus['rows']['border_radius'] : '';

			$rows_box_shadow = isset( $mega_menus['rows']['box_shadow'] ) ? $mega_menus['rows']['box_shadow'] : array();

			$rows_box_shadow_horizontal 	= isset( $rows_box_shadow['horizontal'] ) ? $rows_box_shadow['horizontal'] : '';
			$rows_box_shadow_vertical 	= isset( $rows_box_shadow['vertical'] ) ? $rows_box_shadow['vertical'] : '';
			$rows_box_shadow_blur 		= isset( $rows_box_shadow['blur'] ) ? $rows_box_shadow['blur'] : '';
			$rows_box_shadow_spread 		= isset( $rows_box_shadow['spread'] ) ? $rows_box_shadow['spread'] : '';
			$rows_box_shadow_color 		= isset( $rows_box_shadow['color'] ) ? $rows_box_shadow['color'] : '';


			$rows_box_shadow_css = '';

			if ( $rows_box_shadow_horizontal != '' && $rows_box_shadow_vertical != '' && $rows_box_shadow_blur != '' && $rows_box_shadow_spread != '' && $rows_box_shadow_color != '' ) {
				$rows_box_shadow_css .= $rows_box_shadow_horizontal.'px';
				$rows_box_shadow_css .= ' '.$rows_box_shadow_vertical.'px';
				$rows_box_shadow_css .= ' '.$rows_box_shadow_blur.'px';
				$rows_box_shadow_css .= ' '.$rows_box_shadow_spread.'px';
				$rows_box_shadow_css .= ' '.$rows_box_shadow_color;
			}

			$style = '';
			// rows
			
				$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_row {';
				
					if ( ! empty( $rows_padding ) ) {
						foreach ( $rows_padding as $pos => $val ) {
							if ( $val != '' ) {
								$style .= 'padding-'.$pos.': '.$val.'px;';
							}
						}
					}

					if ( ! empty( $rows_margin ) ) {
						foreach ( $rows_margin as $pos => $val ) {
							if ( $val != '' ) {
								$style .= 'margin-'.$pos.': '.$val.'px;';
							}
						}
					}

					if ( $rows_background != '' ) {
						$style .= 'background-image: linear-gradient('.$rows_background.');';
					}

					if ( $rows_border_top != '' ) {
						$style .= 'border-top-width: '.$rows_border_top.'px;';
					}
					if ( $rows_border_right != '' ) {
						$style .= 'border-right-width: '.$rows_border_right.'px;';
					}
					if ( $rows_border_bottom != '' ) {
						$style .= 'border-bottom-width: '.$rows_border_bottom.'px;';
					}
					if ( $rows_border_left != '' ) {
						$style .= 'border-left-width: '.$rows_border_left.'px;';
					}

					if ( $rows_border_color != '' ) {
						$style .= 'border-color: '.$rows_border_color.';';
					}

					if ( $rows_border_style != '' ) {
						$style .= 'border-style: '.$rows_border_style.';';
					}

					if ( ! empty( $rows_border_radius ) ) {
						foreach ( $rows_border_radius as $pos => $val ) {
							if ( $val != '' ) {
								$style .= 'border-'.$pos.'-radius: '.$val.'px;';
							}
						}
					}

					if ( $rows_box_shadow_css != '' ) {
						$style .= 'box-shadow: '.$rows_box_shadow_css.';';
					}

				$style .= '}';


			return $style;
		}

		public function get_mega_menus_column_style( $mega_menus = array() ){

			$columns_padding = isset( $mega_menus['columns']['padding'] ) ? $mega_menus['columns']['padding'] : array();
			$columns_margin = isset( $mega_menus['columns']['margin'] ) ? $mega_menus['columns']['margin'] : array();
			$columns_background = isset( $mega_menus['columns']['background'] ) ? implode( ', ', array_filter( $mega_menus['columns']['background'] ) ) : '' ;

			$columns_border_top 		= isset( $mega_menus['columns']['border']['top'] ) ? $mega_menus['columns']['border']['top'] : '';
			$columns_border_right 	= isset( $mega_menus['columns']['border']['right'] ) ? $mega_menus['columns']['border']['right'] : '';
			$columns_border_bottom 	= isset( $mega_menus['columns']['border']['bottom'] ) ? $mega_menus['columns']['border']['bottom'] : '';
			$columns_border_left 	= isset( $mega_menus['columns']['border']['left'] ) ? $mega_menus['columns']['border']['left'] : '';
			$columns_border_style 	= isset( $mega_menus['columns']['border']['style'] ) ? $mega_menus['columns']['border']['style'] : '';
			$columns_border_color 	= isset( $mega_menus['columns']['border']['color'] ) ? $mega_menus['columns']['border']['color'] : '';

			$columns_border_radius 	= isset( $mega_menus['columns']['border_radius'] ) ? $mega_menus['columns']['border_radius'] : '';

			$columns_box_shadow = isset( $mega_menus['columns']['box_shadow'] ) ? $mega_menus['columns']['box_shadow'] : array();

			$columns_box_shadow_horizontal 	= isset( $columns_box_shadow['horizontal'] ) ? $columns_box_shadow['horizontal'] : '';
			$columns_box_shadow_vertical 	= isset( $columns_box_shadow['vertical'] ) ? $columns_box_shadow['vertical'] : '';
			$columns_box_shadow_blur 		= isset( $columns_box_shadow['blur'] ) ? $columns_box_shadow['blur'] : '';
			$columns_box_shadow_spread 		= isset( $columns_box_shadow['spread'] ) ? $columns_box_shadow['spread'] : '';
			$columns_box_shadow_color 		= isset( $columns_box_shadow['color'] ) ? $columns_box_shadow['color'] : '';


			$columns_box_shadow_css = '';

			if ( $columns_box_shadow_horizontal != '' && $columns_box_shadow_vertical != '' && $columns_box_shadow_blur != '' && $columns_box_shadow_spread != '' && $columns_box_shadow_color != '' ) {
				$columns_box_shadow_css .= $columns_box_shadow_horizontal.'px';
				$columns_box_shadow_css .= ' '.$columns_box_shadow_vertical.'px';
				$columns_box_shadow_css .= ' '.$columns_box_shadow_blur.'px';
				$columns_box_shadow_css .= ' '.$columns_box_shadow_spread.'px';
				$columns_box_shadow_css .= ' '.$columns_box_shadow_color;
			}


			$columns_sperator_enable = isset( $mega_menus['columns']['sperator']['enable'] ) ? $mega_menus['columns']['sperator']['enable'] : '';
			$columns_sperator_width = isset( $mega_menus['columns']['sperator']['width'] ) ? $mega_menus['columns']['sperator']['width'] : '';
			$columns_sperator_color = isset( $mega_menus['columns']['sperator']['color'] ) ? $mega_menus['columns']['sperator']['color'] : '';
			$columns_sperator_position = isset( $mega_menus['columns']['sperator']['position'] ) ? $mega_menus['columns']['sperator']['position'] : 'right';

			$columns_direction = isset( $mega_menus['columns']['flex_box']['direction'] ) ? $mega_menus['columns']['flex_box']['direction'] : '';
			$columns_justify_content = isset( $mega_menus['columns']['flex_box']['justify_content'] ) ? $mega_menus['columns']['flex_box']['justify_content'] : '';
			$columns_justify_items = isset( $mega_menus['columns']['flex_box']['justify_items'] ) ? $mega_menus['columns']['flex_box']['justify_items'] : '';
			$columns_align_content = isset( $mega_menus['columns']['flex_box']['align_content'] ) ? $mega_menus['columns']['flex_box']['align_content'] : '';
			$columns_align_items = isset( $mega_menus['columns']['flex_box']['align_items'] ) ? $mega_menus['columns']['flex_box']['align_items'] : '';

			

			// columns
			$style = '';

			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_columns {';

				if ( ! empty( $columns_direction ) ) {
					$style .= 'flex-direction: '.$columns_direction.';';
				}

				if ( ! empty( $columns_justify_content ) ) {
					$style .= 'justify-content: '.$columns_justify_content.';';
				}

				if ( ! empty( $columns_justify_items ) ) {
					$style .= 'justify-items: '.$columns_justify_items.';';
				}

				if ( ! empty( $columns_align_content ) ) {
					$style .= 'align-content: '.$columns_align_content.';';
				}

				if ( ! empty( $columns_align_items ) ) {
					$style .= 'align-items: '.$columns_align_items.';';
				}

			$style .= '}';
			


			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_column {';

				if ( ! empty( $columns_padding ) ) {
					foreach ( $columns_padding as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'padding-'.$pos.': '.$val.'px;';
						}
					}
				}

				if ( ! empty( $columns_margin ) ) {
					foreach ( $columns_margin as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'margin-'.$pos.': '.$val.'px;';
						}
					}
				}

				if ( $columns_background != '' ) {
					$style .= 'background-image: linear-gradient('.$columns_background.');';
				}

				if ( $columns_border_top != '' ) {
					$style .= 'border-top-width: '.$columns_border_top.'px;';
				}
				if ( $columns_border_right != '' ) {
					$style .= 'border-right-width: '.$columns_border_right.'px;';
				}
				if ( $columns_border_bottom != '' ) {
					$style .= 'border-bottom-width: '.$columns_border_bottom.'px;';
				}
				if ( $columns_border_left != '' ) {
					$style .= 'border-left-width: '.$columns_border_left.'px;';
				}

				if ( $columns_border_color != '' ) {
					$style .= 'border-color: '.$columns_border_color.';';
				}

				if ( $columns_border_style != '' ) {
					$style .= 'border-style: '.$columns_border_style.';';
				}

				if ( ! empty( $columns_border_radius ) ) {
					foreach ( $columns_border_radius as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'border-'.$pos.'-radius: '.$val.'px;';
						}
					}
				}

				if ( $columns_box_shadow_css != '' ) {
					$style .= 'box-shadow: '.$columns_box_shadow_css.';';
				}

			$style .= '}';
	

			if ( ! empty( $columns_sperator_enable ) ) {
				$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_column:not(:last-child):after {';
					$style .= 'height: 100%;';
					if ( $columns_sperator_width != '' ) {
						$style .= 'width: '.$columns_sperator_width.'px;';
					}

					if ( $columns_sperator_color != '' ) {
						$style .= 'background: '.$columns_sperator_color.';';
					}
					if ( $columns_sperator_position == 'left' ) {
						$style .= 'left: 0;';
						$style .= 'right: auto;';
					} else {
						$style .= 'right: 0;';
						$style .= 'left: auto;';
					}
				$style .= '}';

			}

			return $style;
		}

		public function get_mega_menus_widget_style( $mega_menus = array() ){
			// widgets
			$widget_font 		= isset( $mega_menus['widget']['font_family'] ) ? $mega_menus['widget']['font_family'] : '';
			$widget_font_weight = isset( $mega_menus['widget']['font_weight'] ) ? $mega_menus['widget']['font_weight'] : '';

			$widget_size 		= isset( $mega_menus['widget']['size'] ) ? $mega_menus['widget']['size'] : '';
			$widget_line_height = isset( $mega_menus['widget']['line_height'] ) ? $mega_menus['widget']['line_height'] : '';
			$widget_color 		= isset( $mega_menus['widget']['color'] ) ? $mega_menus['widget']['color'] : '';
			$widget_align 		= isset( $mega_menus['widget']['align'] ) ? $mega_menus['widget']['align'] : '';
			$widget_transform 	= isset( $mega_menus['widget']['transform'] ) ? $mega_menus['widget']['transform'] : '';
			$widget_decoration 	= isset( $mega_menus['widget']['decoration'] ) ? $mega_menus['widget']['decoration'] : '';

			$widget_padding 	= isset( $mega_menus['widget']['padding'] ) ? $mega_menus['widget']['padding'] : '';
			$widget_margin 		= isset( $mega_menus['widget']['margin'] ) ? $mega_menus['widget']['margin'] : '';
			$widget_background 	= isset( $mega_menus['widget']['background'] ) ? implode( ', ', array_filter( $mega_menus['widget']['background'] ) ) : '' ;

			$widget_border_top 		= isset( $mega_menus['widget']['border']['top'] ) ? $mega_menus['widget']['border']['top'] : '';
			$widget_border_right 	= isset( $mega_menus['widget']['border']['right'] ) ? $mega_menus['widget']['border']['right'] : '';
			$widget_border_bottom 	= isset( $mega_menus['widget']['border']['bottom'] ) ? $mega_menus['widget']['border']['bottom'] : '';
			$widget_border_left 	= isset( $mega_menus['widget']['border']['left'] ) ? $mega_menus['widget']['border']['left'] : '';
			$widget_border_style 	= isset( $mega_menus['widget']['border']['style'] ) ? $mega_menus['widget']['border']['style'] : '';
			$widget_border_color 	= isset( $mega_menus['widget']['border']['color'] ) ? $mega_menus['widget']['border']['color'] : '';

			$widget_border_radius 	= isset( $mega_menus['widget']['border_radius'] ) ? $mega_menus['widget']['border_radius'] : '';

			$widget_box_shadow = isset( $mega_menus['widget']['box_shadow'] ) ? $mega_menus['widget']['box_shadow'] : array();

			$widget_box_shadow_horizontal 	= isset( $widget_box_shadow['horizontal'] ) ? $widget_box_shadow['horizontal'] : '';
			$widget_box_shadow_vertical 	= isset( $widget_box_shadow['vertical'] ) ? $widget_box_shadow['vertical'] : '';
			$widget_box_shadow_blur 		= isset( $widget_box_shadow['blur'] ) ? $widget_box_shadow['blur'] : '';
			$widget_box_shadow_spread 		= isset( $widget_box_shadow['spread'] ) ? $widget_box_shadow['spread'] : '';
			$widget_box_shadow_color 		= isset( $widget_box_shadow['color'] ) ? $widget_box_shadow['color'] : '';


			$widget_box_shadow_css = '';

			if ( $widget_box_shadow_horizontal != '' && $widget_box_shadow_vertical != '' && $widget_box_shadow_blur != '' && $widget_box_shadow_spread != '' && $widget_box_shadow_color != '' ) {
				$widget_box_shadow_css .= $widget_box_shadow_horizontal.'px';
				$widget_box_shadow_css .= ' '.$widget_box_shadow_vertical.'px';
				$widget_box_shadow_css .= ' '.$widget_box_shadow_blur.'px';
				$widget_box_shadow_css .= ' '.$widget_box_shadow_spread.'px';
				$widget_box_shadow_css .= ' '.$widget_box_shadow_color;
			}


			/*widgets*/
			$style = '';

			$style .= '.ova-megamenu-item .ova_megamenu .ova-widget {';
				if ( $widget_font != '' ) {
					$style .= 'font-family: '.$widget_font.';';
				}
				if ( $widget_font_weight != '' ) {
					$style .= 'font-weight: '.$widget_font_weight.';';
				}
				if ( $widget_transform != '' ) {
					$style .= 'text-transform: '.$widget_transform.';';
				}
				if ( $widget_decoration != '' ) {
					$style .= 'text-decoration: '.$widget_decoration.';';
				}
				if ( $widget_color != '' ) {
					$style .= 'color: '.$widget_color.';';
				}
				if ( $widget_size != '' ) {
					$style .= 'font-size: '.$widget_size.'px;';
				}
				if ( $widget_line_height != '' ) {
					$style .= 'line-height: '.$widget_line_height.'px;';
				}
				if ( ! empty( $widget_padding ) ) {
					foreach ( $widget_padding as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'padding-'.$pos.': '.$val.'px;';
						}
					}
				}
				if ( ! empty( $widget_margin ) ) {
					foreach ( $widget_margin as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'margin-'.$pos.': '.$val.'px;';
						}
					}
				} else {
					$style .= 'margin: 30px;';
				}

				if ( $widget_background != '' ) {
					$style .= 'background-image: linear-gradient('.$widget_background.');';
				}

				if ( $widget_border_top != '' ) {
					$style .= 'border-top-width: '.$widget_border_top.'px;';
				}
				if ( $widget_border_right != '' ) {
					$style .= 'border-right-width: '.$widget_border_right.'px;';
				}
				if ( $widget_border_bottom != '' ) {
					$style .= 'border-bottom-width: '.$widget_border_bottom.'px;';
				}
				if ( $widget_border_left != '' ) {
					$style .= 'border-left-width: '.$widget_border_left.'px;';
				}

				if ( $widget_border_color != '' ) {
					$style .= 'border-color: '.$widget_border_color.';';
				}

				if ( $widget_border_style != '' ) {
					$style .= 'border-style: '.$widget_border_style.';';
				}

				if ( ! empty( $widget_border_radius ) ) {
					foreach ( $widget_border_radius as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'border-'.$pos.'-radius: '.$val.'px;';
						}
					}
				}

				if ( $widget_box_shadow_css != '' ) {
					$style .= 'box-shadow: '.$widget_box_shadow_css.';';
				}

				if ( $widget_align != '' ) {
					$style .= 'text-align: '.$widget_align.';';
				}

			$style .= '}';

			return $style;
		}

		public function get_mega_menus_heading_style( $mega_menus = array() ){

			/* heading */
			$heading_font 			= isset( $mega_menus['heading']['font_family'] ) ? $mega_menus['heading']['font_family'] : '';
			$heading_font_weight 	= isset( $mega_menus['heading']['font_weight'] ) ? $mega_menus['heading']['font_weight'] : '';

			$heading_size 			= isset( $mega_menus['heading']['size'] ) ? $mega_menus['heading']['size'] : '';
			$line_height 	= isset( $mega_menus['heading']['line_height'] ) ? $mega_menus['heading']['line_height'] : '';
			$heading_color 			= isset( $mega_menus['heading']['color'] ) ? $mega_menus['heading']['color'] : '';
			$heading_align 			= isset( $mega_menus['heading']['align'] ) ? $mega_menus['heading']['align'] : '';
			$heading_transform 		= isset( $mega_menus['heading']['transform'] ) ? $mega_menus['heading']['transform'] : '';
			$heading_decoration 	= isset( $mega_menus['heading']['decoration'] ) ? $mega_menus['heading']['decoration'] : '';

			$heading_padding 	= isset( $mega_menus['heading']['padding'] ) ? $mega_menus['heading']['padding'] : '';
			$heading_margin 	= isset( $mega_menus['heading']['margin'] ) ? $mega_menus['heading']['margin'] : '';
			$heading_background = isset( $mega_menus['heading']['background'] ) ? implode( ', ', array_filter( $mega_menus['heading']['background'] ) ) : '' ;

			$heading_border_top 	= isset( $mega_menus['heading']['border']['top'] ) ? $mega_menus['heading']['border']['top'] : '';
			$heading_border_right 	= isset( $mega_menus['heading']['border']['right'] ) ? $mega_menus['heading']['border']['right'] : '';
			$heading_border_bottom 	= isset( $mega_menus['heading']['border']['bottom'] ) ? $mega_menus['heading']['border']['bottom'] : '';
			$heading_border_left 	= isset( $mega_menus['heading']['border']['left'] ) ? $mega_menus['heading']['border']['left'] : '';
			$heading_border_style 	= isset( $mega_menus['heading']['border']['style'] ) ? $mega_menus['heading']['border']['style'] : '';
			$heading_border_color 	= isset( $mega_menus['heading']['border']['color'] ) ? $mega_menus['heading']['border']['color'] : '';

			$heading_border_radius 	= isset( $mega_menus['heading']['border_radius'] ) ? $mega_menus['heading']['border_radius'] : '';

			$heading_box_shadow 	= isset( $mega_menus['heading']['box_shadow'] ) ? $mega_menus['heading']['box_shadow'] : '';

			$heading_box_shadow_css = '';
			$heading_box_shadow_horizontal 	= isset( $mega_menus['heading']['box_shadow']['horizontal'] ) ? $mega_menus['heading']['box_shadow']['horizontal'] : '';
			$heading_box_shadow_vertical 	= isset( $mega_menus['heading']['box_shadow']['vertical'] ) ? $mega_menus['heading']['box_shadow']['vertical'] : '';
			$heading_box_shadow_blur 		= isset( $mega_menus['heading']['box_shadow']['blur'] ) ? $mega_menus['heading']['box_shadow']['blur'] : '';
			$heading_box_shadow_spread 		= isset( $mega_menus['heading']['box_shadow']['spread'] ) ? $mega_menus['heading']['box_shadow']['spread'] : '';
			$heading_box_shadow_color 		= isset( $mega_menus['heading']['box_shadow']['color'] ) ? $mega_menus['heading']['box_shadow']['color'] : '';

			if ( $heading_box_shadow_horizontal != '' && $heading_box_shadow_vertical != '' ) {
				$heading_box_shadow_css = $heading_box_shadow_horizontal.'px '.$heading_box_shadow_vertical.'px';
				if ( $heading_box_shadow_blur != '' ) {
					$heading_box_shadow_css .= ' '.$heading_box_shadow_blur.'px';
				}
				if ( $heading_box_shadow_spread != '' ) {
					$heading_box_shadow_css .= ' '.$heading_box_shadow_spread.'px';
				}
				if ( $heading_box_shadow_color != '' ) {
					$heading_box_shadow_css .= ' '.$heading_box_shadow_color;
				}
			}


			$heading_line_enable 	= isset( $mega_menus['heading']['line']['enable'] ) ? $mega_menus['heading']['line']['enable'] : '';
			$heading_line_top 		= isset( $mega_menus['heading']['line']['top'] ) ? $mega_menus['heading']['line']['top'] : '';
			$heading_line_width 	= isset( $mega_menus['heading']['line']['width'] ) ? $mega_menus['heading']['line']['width'] : '';
			$heading_line_height 	= isset( $mega_menus['heading']['line']['height'] ) ? $mega_menus['heading']['line']['height'] : '';
			$heading_line_color 	= isset( $mega_menus['heading']['line']['color'] ) ? $mega_menus['heading']['line']['color'] : '';

			/*heading*/
			$style = '';

			$style .= '.ova-megamenu-item .ova_megamenu .ova-heading {';
				if ( $heading_font != '' ) {
					$style .= 'font-family: '.$heading_font.';';
				}
				if ( $heading_font_weight != '' ) {
					$style .= 'font-weight: '.$heading_font_weight.';';
				}
				if ( $heading_transform != '' ) {
					$style .= 'text-transform: '.$heading_transform.';';
				}
				if ( $heading_decoration != '' ) {
					$style .= 'text-decoration: '.$heading_decoration.';';
				}
				if ( $heading_color != '' ) {
					$style .= 'color: '.$heading_color.';';
				}
				if ( $heading_size != '' ) {
					$style .= 'font-size: '.$heading_size.'px;';
				}
				if ( $line_height != '' ) {
					$style .= 'line-height: '.$line_height.'px;';
				}
				if ( ! empty( $heading_padding ) ) {
					foreach ( $heading_padding as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'padding-'.$pos.': '.$val.'px;';
						}
					}
				}
				if ( ! empty( $heading_margin ) ) {
					foreach ( $heading_margin as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'margin-'.$pos.': '.$val.'px;';
						}
					}
				}

				if ( $heading_background != '' ) {
					$style .= 'background-image: linear-gradient('.$heading_background.');';
				}

				if ( $heading_border_top != '' ) {
					$style .= 'border-top-width: '.$heading_border_top.'px;';
				}
				if ( $heading_border_right != '' ) {
					$style .= 'border-right-width: '.$heading_border_right.'px;';
				}
				if ( $heading_border_bottom != '' ) {
					$style .= 'border-bottom-width: '.$heading_border_bottom.'px;';
				}
				if ( $heading_border_left != '' ) {
					$style .= 'border-left-width: '.$heading_border_left.'px;';
				}

				if ( $heading_border_color != '' ) {
					$style .= 'border-color: '.$heading_border_color.';';
				}

				if ( $heading_border_style != '' ) {
					$style .= 'border-style: '.$heading_border_style.';';
				}

				if ( ! empty( $heading_border_radius ) ) {
					foreach ( $heading_border_radius as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'border-'.$pos.'-radius: '.$val.'px;';
						}
					}
				}

				if ( $heading_box_shadow_css != '' ) {
					$style .= 'box-shadow: '.$heading_box_shadow_css.';';
				}

			$style .= '}';

			if ( $heading_align != '' ) {

				$style .= '.ova-megamenu-item .ova_megamenu .ova-heading {';
					$style .= 'text-align: '.$heading_align.';';
				$style .= '}';

				if ( $heading_align == 'right' ) {
					$style .= '.ova-megamenu-item .ova_megamenu .ova-heading:after {';
						$style .= 'left: auto;';
						$style .= 'right: 0;';
					$style .= '}';
				}
				
			}

			if ( ! empty( $heading_line_enable ) ) {
				$style .= '.ova-megamenu-item .ova_megamenu .ova-heading:after {';
					if ( $heading_line_top != '' ) {
						$style .= 'bottom: -'.$heading_line_top.'px;';
					}
					if ( $heading_line_width != '' ) {
						$style .= 'width: '.$heading_line_width.'px;';
					}
					if ( $heading_line_height != '' ) {
						$style .= 'height: '.$heading_line_height.'px;';
					}
					if ( $heading_line_color != '' ) {
						$style .= 'background: '.$heading_line_color.';';
					}
				$style .= '}';
			}

			return $style;
		}

		public function get_mega_menus_menu_item_style( $mega_menus = array() ){

			/* menu item */
			$menu_item_font = isset( $mega_menus['menu_item']['font_family'] ) ? $mega_menus['menu_item']['font_family'] : '';
			$menu_item_font_weight = isset( $mega_menus['menu_item']['font_weight'] ) ? $mega_menus['menu_item']['font_weight'] : '';


			$menu_item_size = isset( $mega_menus['menu_item']['size'] ) ? $mega_menus['menu_item']['size'] : '';
			$menu_item_line_height = isset( $mega_menus['menu_item']['line_height'] ) ? $mega_menus['menu_item']['line_height'] : '';
			$menu_item_color = isset( $mega_menus['menu_item']['color'] ) ? $mega_menus['menu_item']['color'] : '#000';
			$menu_item_align = isset( $mega_menus['menu_item']['align'] ) ? $mega_menus['menu_item']['align'] : '';
			$menu_item_transform = isset( $mega_menus['menu_item']['transform'] ) ? $mega_menus['menu_item']['transform'] : '';
			$menu_item_decoration = isset( $mega_menus['menu_item']['decoration'] ) ? $mega_menus['menu_item']['decoration'] : '';

			$menu_item_hover = isset( $mega_menus['menu_item']['hover'] ) ? $mega_menus['menu_item']['hover'] : '';
			$menu_item_active = isset( $mega_menus['menu_item']['active'] ) ? $mega_menus['menu_item']['active'] : '';

			$menu_item_padding = isset( $mega_menus['menu_item']['padding'] ) ? $mega_menus['menu_item']['padding'] : '';
			$menu_item_margin = isset( $mega_menus['menu_item']['margin'] ) ? $mega_menus['menu_item']['margin'] : '';
			$menu_item_background = isset( $mega_menus['menu_item']['background'] ) ? implode( ', ', array_filter( $mega_menus['menu_item']['background'] ) ) : '' ;
			$menu_item_background_hover = isset( $mega_menus['menu_item']['background_hover'] ) ? implode( ', ', array_filter( $mega_menus['menu_item']['background_hover'] ) ) : '' ;
			$menu_item_background_active = isset( $mega_menus['menu_item']['background_active'] ) ? implode( ', ', array_filter( $mega_menus['menu_item']['background_active'] ) ) : '' ;

			$menu_item_border_top = isset( $mega_menus['menu_item']['border']['top'] ) ? $mega_menus['menu_item']['border']['top'] : '';
			$menu_item_border_right = isset( $mega_menus['menu_item']['border']['right'] ) ? $mega_menus['menu_item']['border']['right'] : '';
			$menu_item_border_bottom = isset( $mega_menus['menu_item']['border']['bottom'] ) ? $mega_menus['menu_item']['border']['bottom'] : '';
			$menu_item_border_left = isset( $mega_menus['menu_item']['border']['left'] ) ? $mega_menus['menu_item']['border']['left'] : '';
			$menu_item_border_style = isset( $mega_menus['menu_item']['border']['style'] ) ? $mega_menus['menu_item']['border']['style'] : '';
			$menu_item_border_color = isset( $mega_menus['menu_item']['border']['color'] ) ? $mega_menus['menu_item']['border']['color'] : '';

			$menu_item_border_radius = isset( $mega_menus['menu_item']['border_radius'] ) ? $mega_menus['menu_item']['border_radius'] : '';

			$menu_item_box_shadow = isset( $mega_menus['menu_item']['box_shadow'] ) ? $mega_menus['menu_item']['box_shadow'] : '';

			$menu_item_box_shadow_css = '';
			$menu_item_box_shadow_horizontal 	= isset( $mega_menus['menu_item']['box_shadow']['horizontal'] ) ? $mega_menus['menu_item']['box_shadow']['horizontal'] : '';
			$menu_item_box_shadow_vertical 	= isset( $mega_menus['menu_item']['box_shadow']['vertical'] ) ? $mega_menus['menu_item']['box_shadow']['vertical'] : '';
			$menu_item_box_shadow_blur 		= isset( $mega_menus['menu_item']['box_shadow']['blur'] ) ? $mega_menus['menu_item']['box_shadow']['blur'] : '';
			$menu_item_box_shadow_spread 		= isset( $mega_menus['menu_item']['box_shadow']['spread'] ) ? $mega_menus['menu_item']['box_shadow']['spread'] : '';
			$menu_item_box_shadow_color 		= isset( $mega_menus['menu_item']['box_shadow']['color'] ) ? $mega_menus['menu_item']['box_shadow']['color'] : '';

			if ( $menu_item_box_shadow_horizontal != '' && $menu_item_box_shadow_vertical != '' ) {
				$menu_item_box_shadow_css = $menu_item_box_shadow_horizontal.'px '.$menu_item_box_shadow_vertical.'px';
				if ( $menu_item_box_shadow_blur != '' ) {
					$menu_item_box_shadow_css .= ' '.$menu_item_box_shadow_blur.'px';
				}
				if ( $menu_item_box_shadow_spread != '' ) {
					$menu_item_box_shadow_css .= ' '.$menu_item_box_shadow_spread.'px';
				}
				if ( $menu_item_box_shadow_color != '' ) {
					$menu_item_box_shadow_css .= ' '.$menu_item_box_shadow_color;
				}
			}

			/* menu item */
			$style = '';

			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_item {';
				if ( $menu_item_line_height != '' ) {
					$style .= 'line-height: '.$menu_item_line_height.'px;';
				}
			$style .= '}';

			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_item a {';

				if ( $menu_item_font != '' ) {
					$style .= 'font-family: '.$menu_item_font.';';
				}
				if ( $menu_item_line_height != '' ) {
					$style .= 'line-height: '.$menu_item_line_height.'px;';
				}
				if ( $menu_item_font_weight != '' ) {
					$style .= 'font-weight: '.$menu_item_font_weight.';';
				}
				if ( $menu_item_transform != '' ) {
					$style .= 'text-transform: '.$menu_item_transform.';';
				}
				if ( $menu_item_decoration != '' ) {
					$style .= 'text-decoration: '.$menu_item_decoration.';';
				}
				if ( $menu_item_color != '' ) {
					$style .= 'color: '.$menu_item_color.';';
				}
				if ( $menu_item_size != '' ) {
					$style .= 'font-size: '.$menu_item_size.'px;';
				}
				if ( ! empty( $menu_item_padding ) ) {
					foreach ( $menu_item_padding as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'padding-'.$pos.': '.$val.'px;';
						}
					}
				}
				if ( ! empty( $menu_item_margin ) ) {
					foreach ( $menu_item_margin as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'margin-'.$pos.': '.$val.'px;';
						}
					}
				}
				if ( $menu_item_background != '' ) {
					$style .= 'background-image: linear-gradient('.$menu_item_background.');';
				}

				if ( $menu_item_border_top != '' ) {
					$style .= 'border-top-width: '.$menu_item_border_top.'px;';
				}
				if ( $menu_item_border_right != '' ) {
					$style .= 'border-right-width: '.$menu_item_border_right.'px;';
				}
				if ( $menu_item_border_bottom != '' ) {
					$style .= 'border-bottom-width: '.$menu_item_border_bottom.'px;';
				}
				if ( $menu_item_border_left != '' ) {
					$style .= 'border-left-width: '.$menu_item_border_left.'px;';
				}
				if ( $menu_item_border_color != '' ) {
					$style .= 'border-color: '.$menu_item_border_color.';';
				}

				if ( $menu_item_border_style != '' ) {
					$style .= 'border-style: '.$menu_item_border_style.';';
				}

				if ( ! empty( $menu_item_border_radius ) ) {
					foreach ( $menu_item_border_radius as $pos => $val ) {
						if ( $val != '' ) {
							$style .= 'border-'.$pos.'-radius: '.$val.'px;';
						}
					}
				}

				if ( $menu_item_box_shadow_css != '' ) {
					$style .= 'box-shadow: '.$menu_item_box_shadow_css.';';
				}
				
			$style .= '}';

			
			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_item a:hover {';
				if ( $menu_item_hover != '' ) {
					$style .= 'color: '.$menu_item_hover.';';
				}
				if ( $menu_item_background_hover != '' ) {
					$style .= 'background-image: linear-gradient('.$menu_item_background_hover.');';
				}
			$style .= '}';

			$style .= '.ova-megamenu-item .ova_megamenu .ova_megamenu_item.active a {';
				if ( $menu_item_active != '' ) {
					$style .= 'color: '.$menu_item_active.';';
				}
				if ( $menu_item_background_active != '' ) {
					$style .= 'background-image: linear-gradient('.$menu_item_background_active.');';
				}
			$style .= '}';

			return $style;
		}

		public function get_font_face_style( $ova_megamenu_opts = array() ){
			$mega_menus = isset( $ova_megamenu_opts['mega_menus'] ) ? $ova_megamenu_opts['mega_menus'] : array();
			$google_fonts = $this->google_fonts;

			// widgets
			$widget_font 			= isset( $mega_menus['widget']['font_family'] ) ? $mega_menus['widget']['font_family'] : '';
			$widget_font_weight 	= isset( $mega_menus['widget']['font_weight'] ) ? $mega_menus['widget']['font_weight'] : '';
			$widget_font_src 		= $google_fonts->get_font_src( $widget_font, $widget_font_weight );

			/* heading */
			$heading_font 			= isset( $mega_menus['heading']['font_family'] ) ? $mega_menus['heading']['font_family'] : '';
			$heading_font_weight 	= isset( $mega_menus['heading']['font_weight'] ) ? $mega_menus['heading']['font_weight'] : '';
			$heading_font_src 		= $google_fonts->get_font_src( $heading_font, $heading_font_weight );

			/* menu item */
			$menu_item_font 		= isset( $mega_menus['menu_item']['font_family'] ) ? $mega_menus['menu_item']['font_family'] : '';
			$menu_item_font_weight 	= isset( $mega_menus['menu_item']['font_weight'] ) ? $mega_menus['menu_item']['font_weight'] : '';
			$menu_item_font_src 	= $google_fonts->get_font_src( $menu_item_font, $menu_item_font_weight );

			/*font face*/
			$style = '';

			if ( $widget_font != '' ) {
				$style .= '@font-face {';
					$style .= 'font-family: '.$widget_font.';';

					if ( $widget_font_src != '' ) {
						$style .= 'src: url('.$widget_font_src.');';
					}

					if ( $widget_font_weight != '' ) {
						$style .= 'font-weight: '.$widget_font_weight.';';
					}
					
				$style .= '}';
			}

			if ( $heading_font != '' ) {
				$style .= '@font-face {';
					$style .= 'font-family: '.$heading_font.';';

					if ( $heading_font_src != '' ) {
						$style .= 'src: url('.$heading_font_src.');';
					}

					if ( $heading_font_weight != '' ) {
						$style .= 'font-weight: '.$heading_font_weight.';';
					}
					
				$style .= '}';
			}

			if ( $menu_item_font != '' ) {
				$style .= '@font-face {';
					$style .= 'font-family: '.$menu_item_font.';';

					if ( $menu_item_font_src != '' ) {
						$style .= 'src: url('.$menu_item_font_src.');';
					}

					if ( $menu_item_font_weight != '' ) {
						$style .= 'font-weight: '.$menu_item_font_weight.';';
					}
					
				$style .= '}';
			}

			return $style;
		}

		public function get_setting_css(){
			$ova_megamenu_opts = get_option( 'ova_megamenu', array() );

			/* STYLE START */
			$style = '';

			/* @font-face */
			$style .= $this->get_font_face_style( $ova_megamenu_opts );

			/* GENERAL */
			$style .= $this->get_general_style( $ova_megamenu_opts );

			/* MEGA MENUS */
			$style .= $this->get_mega_menus_style( $ova_megamenu_opts );

			return $style;
		}
	}
}