<?php
if ( !defined( 'ABSPATH' ) ) exit();


if ( ! function_exists('ova_megamenu_get_shortcode_by_id') ) {
	function ova_megamenu_get_shortcode_by_id( $id ){
		$shortcode = '[ova-megamenu id="'.esc_attr( $id ).'"]';
		return $shortcode;
	}
}

if ( ! function_exists('ova_megamenu_get_type_item') ) {
	function ova_megamenu_get_type_item(){
		return array(
			'text' => esc_html__( 'Text', 'ova-megamenu' ),
			'shortcode' => esc_html__( 'Shortcode', 'ova-megamenu' ),
			'menu' => esc_html__( 'Menu', 'ova-megamenu' ),
		);
	}
}

if ( ! function_exists('ova_megamenu_get_menus') ) {
	function ova_megamenu_get_menus(){
		$args = array( 'hide_empty' => false );
		return get_terms( 'nav_menu', apply_filters( 'ova_megamenu_get_menus', $args ) );
	}
}

if ( ! function_exists('ova_megamenu_get_page_by_title') ) {
	function ova_megamenu_get_page_by_title( $title ){
		if ( empty( $title ) ) {
			return false;
		}
		$args = array(
			'post_type' => 'page',
			'post_status' => 'publish',
			's' => $title,
			'posts_per_page' => -1,
		);

		$pages = new WP_Query( $args );
		return $pages;
	}
}

if ( ! function_exists('ova_megamenu_get_page_default') ) {
	function ova_megamenu_get_page_default(){
		$args = array(
			'number' => 5,
		);
		$pages = get_pages( apply_filters( 'ova_megamenu_get_page_default', $args ) );
		return $pages;
	}
}

if ( ! function_exists('ova_megamenu_get_post_default') ) {
	function ova_megamenu_get_post_default(){
		$args = array(
			'number' => 5,
		);
		$posts = get_posts( apply_filters( 'ova_megamenu_get_post_default', $args ) );
		return $posts;
	}
}

if ( ! function_exists('ova_megamenu_get_post_by_title') ) {
	function ova_megamenu_get_post_by_title( $title ){
		if ( empty( $title ) ) {
			return false;
		}
		$args = array(
			'post_type' => 'post',
			'post_status' => 'publish',
			's' => $title,
			'posts_per_page' => -1,
		);

		$posts = new WP_Query( $args );
		return $posts;
	}
}

if ( ! function_exists('ova_megamenu_get_custom_post_by_title') ) {
	function ova_megamenu_get_custom_post_by_title( $title, $post_type ){
		if ( empty( $title ) || empty( $post_type ) ) {
			return false;
		}
		$args = array(
			'post_type' => $post_type,
			'post_status' => 'publish',
			's' => $title,
			'posts_per_page' => -1,
		);
		$custom_posts = new WP_Query( $args );
		return $custom_posts;
	}
}

if ( ! function_exists('ova_megamenu_get_categories_default') ) {
	function ova_megamenu_get_categories_default(){
		$args = array(
			'taxonomy' => 'category',
			'number' => 5,
			'hide_empty' => false,
		);

		$categories = get_terms( apply_filters( 'ova_megamenu_get_categories_default', $args ) );
		return $categories;
	}
}

if ( ! function_exists('ova_megamenu_get_custom_taxonomies') ) {
	function ova_megamenu_get_custom_taxonomies(){
		$args = array(
			'public'   => true,
			'_builtin' => false,
			'show_in_nav_menus' => true,
		);
		$taxonomies = get_taxonomies( apply_filters( 'ova_megamenu_get_custom_taxonomies', $args ), 'objects' );
		return $taxonomies;
	}
}

if ( ! function_exists('ova_megamenu_get_custom_post_types') ) {
	function ova_megamenu_get_custom_post_types(){
		$args = array(
	       'public'   => true,
	       '_builtin' => false,
	       'show_in_nav_menus' => true,
	    );
	    $post_types = get_post_types( apply_filters( 'ova_megamenu_get_custom_post_types', $args ) , 'objects');
	    return $post_types;
	}
}

if ( ! function_exists('ova_megamenu_get_terms_by_name') ) {
	function ova_megamenu_get_terms_by_name( $name, $taxonomy = 'category' ){
		if ( empty( $name ) ) {
			return false;
		}
		$args = array(
			'taxonomy' => $taxonomy,
			'hide_empty' => false,
			'name__like' => $name,
		);
		$terms = get_terms( apply_filters( 'ova_megamenu_get_terms_by_name', $args ) );
		return $terms;
	}
}

if ( ! function_exists('ova_megamenu_get_custom_taxonomies_default') ) {
	function ova_megamenu_get_custom_taxonomies_default( $taxonomy_name ){
		if ( empty( $taxonomy_name ) ) {
			return false;
		}
		$args = array(
			'taxonomy' => $taxonomy_name,
			'hide_empty' => false,
			'number' => 5,
		);
		$terms = get_terms( apply_filters( 'ova_megamenu_get_custom_taxonomies_default', $args ) );
		return $terms;
	}
}

if ( ! function_exists('ova_megamenu_get_custom_post_type_default') ) {
	function ova_megamenu_get_custom_post_type_default( $post_type ){
		if ( empty( $post_type ) ) {
			return false;
		}

		$args = array(
			'post_type'   => $post_type,
			'post_status' => 'publish',
			'posts_per_page' => 5,
		);
		
		$custom_posts = new WP_Query( apply_filters( 'ova_megamenu_get_custom_post_type_default', $args ) );
		return $custom_posts;
	}
}

if ( ! function_exists('ova_recursive_sanitize_text_field') ) {
	function ova_recursive_sanitize_text_field($input) {
		if ( is_array( $input ) ) {
			foreach ( $input as $key => &$value ) {
		        if ( is_array( $value ) ) {
		            $value = ova_recursive_sanitize_text_field($value);
		        }
		        else {
		            $value = sanitize_text_field( $value );
		        }
		    }
		} else {
			$input = sanitize_text_field( $input );
		}
		
	    return $input;
	}
}

if ( ! function_exists('ova_megamenu_recursive_sanitize_metabox') ) {
	function ova_megamenu_recursive_sanitize_metabox($input) {
		if ( is_array( $input ) ) {
			foreach ( $input as $key => &$value ) {

				if ( $key === 'text' ) {
					$value = wp_kses_post( $value );
					continue;
				}
				if ( $key === 'url' ) {
					$value = sanitize_url( $value );
					continue;
				}

				if ( is_array( $value ) ) {
		            $value = ova_megamenu_recursive_sanitize_metabox($value);
		        }
		        else {
		            $value = sanitize_text_field( $value );
		        }

		        
		    }
		} else {
			$input = sanitize_text_field( $input );
		}
		
	    return $input;
	}
}

if ( ! function_exists('ova_megamenu_get_attr_menu_item') ) {
	function ova_megamenu_get_attr_menu_item( $target = '', $title = '' ){
		$attributes = array();
		$output = '';
		if ( ! empty( $target ) ) {
			$attributes[] = 'target="'.$target.'"';
		}
		if ( ! empty( $title ) ) {
			$attributes[] = 'title="'.$title.'"';
		}
		if ( ! empty( $attributes ) ) {
			$output = implode(' ', $attributes);
		}
		return $output;
	}
}

if ( ! function_exists('ova_megamenu_get_the_content') ) {
	function ova_megamenu_get_the_content( $menu_id ){
		$args = array(
			'post_type' => 'ova-megamenu',
			'p' => $menu_id,
			'post_status' => 'publish'
		);
		$the_query = new WP_Query( $args );
			// The Loop
		ob_start();
			if ( $the_query->have_posts() ) :
				while ( $the_query->have_posts() ) : $the_query->the_post();
				// Do Stuff
				the_content();
				endwhile;
			endif;

		// Reset Post Data
		wp_reset_postdata();
		$output = ob_get_clean();
		return $output;
	}
}

if ( ! function_exists('ova_megamenu_strip_all_tags') ) {
	function ova_megamenu_strip_all_tags( $input ){
		if ( is_array( $input ) ) {
			foreach ( $input as $key => $value ) {
				$value = wp_strip_all_tags( $value );
			}
		} else {
			$input = wp_strip_all_tags( $input );
		}
		return $input;
	}
}

if ( ! function_exists('ova_megamenu_google_font_orderby') ) {
	function ova_megamenu_google_font_orderby(){
		return apply_filters( 'ova_megamenu_google_font_orderby', 'popular' );
	}
}

if ( ! function_exists('ova_megamenu_google_font_count') ) {
	function ova_megamenu_google_font_count(){
		return apply_filters( 'ova_megamenu_google_font_count', 'all' );
	}
}


if ( ! function_exists('ova_megamenu_get_tab_content') ) {
	function ova_megamenu_get_tab_content( $data_tab_content = array(), $tab_name = null ){
		if ( ! empty( $data_tab_content ) && ! empty( $tab_name ) ) {

			$widgets = ova_megamenu_get_type_item();

			foreach ( $data_tab_content as $k => $row ): ?>

				<li class="ova_megamenu_row card" data-row="<?php echo esc_attr( $k ); ?>">
					<ul class="ova_megamenu_columns">
						<?php if ( $row ): ?>
							<?php foreach ( $row as $j => $value ): ?>
								<?php $column_width = isset( $value['column_width'] ) ? $value['column_width'] : ''; ?>
								<li class="ova_megamenu_column card"
									style="width: <?php echo esc_attr( $column_width.'%' ); ?>;"
									data-column="<?php echo esc_attr( $j ); ?>"
									data-row="<?php echo esc_attr( $k ); ?>">
									<button type="button" class="ova_megamenu_remove_column button button-small button-secondary">&#x2715;</button>
									<button type="button" class="ova_megamenu_edit_width button button-small button-secondary"><span class="dashicons dashicons-edit"></span></button>
									<div class="ova_megamenu_resize"></div>
									
									<div class="show_column_percent"><?php echo esc_html( $column_width.'%' ); ?></div>
									<input type="number" name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][column_width]" class="ova_megamenu_column_width" min="1" step="0.1" value="<?php echo esc_attr( $column_width ); ?>">
									<ul class="ova_megamenu_widgets">
										<?php
											$widget_items = isset( $value['widget'] ) ? $value['widget'] : array();
										?>
										<?php if ( ! empty( $widget_items ) ): ?>
											<?php foreach ($widget_items as $key => $val): ?>
												
												<li class="ova_megamenu_widget_item card" data-widget="<?php echo esc_attr( $key ); ?>" data-column="<?php echo esc_attr( $j ); ?>" data-row="<?php echo esc_attr( $k ); ?>">

														<button type="button" class="ova_megamenu_remove_widget button button-small button-primary">&#x2715;</button>
														<table class="form-table">
														<tbody>

														<tr>
															<?php
															$widget_type = isset( $val['type'] ) ? $val['type'] : '';
															?>
															<td><label for=""><strong><?php esc_html_e( 'Choose a type', 'ova-megamenu' ); ?></strong></label>
																<select class="ova_megamenu_widget" name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][type]" id="">
																	<option selected="selected" value=""><?php esc_html_e( '--- Type ---', 'ova-megamenu' ); ?></option>
																	<?php if ( ! empty( $widgets ) ): ?>
																		<?php foreach ($widgets as $y => $type): ?>
																			<option <?php selected( $widget_type, $y ); ?> value="<?php echo esc_attr( $y ); ?>"><?php echo esc_html( $type ); ?></option>
																		<?php endforeach; ?>
																	<?php endif; ?>
																</select>
															</td>
														</tr>


														<?php $heading = isset( $val['heading'] ) ? $val['heading'] : ''; ?>
														<tr>
															<td><label for=""><strong><?php esc_html_e( 'Heading', 'ova-megamenu' ); ?></strong></label>
															<input name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][heading]" type="text" placeholder="<?php esc_attr_e( 'Enter heading', 'ova-megamenu' ); ?>" id="" value="<?php echo esc_attr( $heading ); ?>" class="large-text"></td>
														</tr>

														<?php  $shortcode = isset( $val['shortcode'] ) ? $val['shortcode'] : '';  ?>
														<tr class="ova_megamenu_field <?php echo $widget_type == 'shortcode' ? 'active' : ''; ?>" data-id="shortcode">
															<td><label for=""><strong><?php esc_html_e( 'Shortcode', 'ova-megamenu' ); ?></strong></label>
															<input name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][shortcode]" type="text" placeholder="<?php esc_attr_e( 'Enter shortcode', 'ova-megamenu' ); ?>" id="" value="<?php echo esc_attr( $shortcode ); ?>" class="large-text"></td>
														</tr>

															<tr class="ova_megamenu_field <?php echo $widget_type == 'text' ? 'active' : ''; ?>" data-id="text">
																<td>
																	<label for=""><strong><?php esc_html_e( 'Text', 'ova-megamenu' ); ?></strong></label>
																	<?php
																		$settings  = array(
																			'textarea_name' => 'ova_megamenu_'.$tab_name.'['.$k.']['.$j.'][widget]['.$key.'][text]',
																			'textarea_rows' => 10,
																			'editor_height' => 230,
																			'wpautop' => false,
																			'quicktags' => true,
																		);
																		$content = isset( $val['text'] ) ? $val['text'] : '';
																		$id = 'ova_megamenu_'.uniqid();
																		wp_editor( wpautop($content), $id, $settings );
																	?>
																	<script type="text/javascript">
																		var id = "<?php echo $id; ?>";
																		tinymce.execCommand( 'mceAddEditor', true, id );
																		quicktags({id: id});
																	</script>
																</td>
															</tr>
															<?php
															$menu_type = isset( $val['menu_type'] ) ? $val['menu_type'] : 'add-item';
															?>
															<tr class="ova_megamenu_field <?php echo $widget_type == 'menu' ? 'active' : ''; ?>" data-id="menu">
																<td>
																	<label for=""><?php esc_html_e( 'Type:', 'ova-megamenu' ); ?></label>
																	
																	<span><input  class="menu_type" name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_type]" <?php checked( $menu_type, 'add-item') ?> type="radio" value="add-item">
																		<?php esc_html_e( 'Add items', 'ova-megamenu' ); ?>
																	</span>

																	<span><input class="menu_type" name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_type]" <?php checked( $menu_type, 'select-menu' ); ?> type="radio" value="select-menu">
																		<?php esc_html_e( 'Select menu', 'ova-megamenu' ); ?>
																	</span>
																</td>

															</tr>

															<tr class="ova_megamenu_sub_field <?php echo $widget_type == 'menu' && $menu_type == 'select-menu' ? 'active' : ''; ?>" data-id="select-menu">
																<?php
																$nav_menus = ova_megamenu_get_menus();
																$menu_selected = isset( $val['menu'] ) ? $val['menu'] : '';
																?>
																<td>
																	<label for=""><strong><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></strong></label>
																
																	<select name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu]" id="">
																		<option value=""><?php esc_html_e( 'Choose a menu', 'ova-megamenu' ); ?></option>
																		<?php if ( ! empty( $nav_menus ) ): ?>
																			<?php foreach ($nav_menus as $obj): ?>
																				<option
																				<?php selected( $menu_selected, $obj->term_id ); ?>
																				value="<?php echo esc_attr( $obj->term_id ); ?>"><?php echo esc_html( $obj->name ); ?></option>
																			<?php endforeach; ?>
																		<?php endif; ?>
																	</select>
																</td>
															</tr>

															<tr class="ova_megamenu_sub_field <?php echo $widget_type == 'menu' && $menu_type == 'add-item' ? 'active' : ''; ?>" data-id="add-item">
																
																<td>
																	<?php
																	$menu_items = isset( $val['menu_item'] ) ? $val['menu_item'] : array();
																	$active_bulk_select = ! empty( $menu_items ) ? 'is-active' : '';
																	?>

																	<div class="ova_header">

																			<button type="button" class="
																			<?php echo esc_attr( $active_bulk_select ); ?> ova_megamenu_menu_item_bulk_select button button-small button-secondary">
																				<input type="checkbox" class="menu_item_bulk_select" value="1">
																				<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
																			</button>

																	</div>
																	<ul class="list-menu-item">
																		<?php if ( ! empty( $menu_items ) ): ?>
																			<?php foreach ( $menu_items as $k_item => $menu_item ):
																				$label = isset( $menu_item['label'] ) ? $menu_item['label'] : '';
																				$title = isset( $menu_item['title'] ) ? $menu_item['title'] : '';
																				$class = isset( $menu_item['class'] ) ? $menu_item['class'] : '';
																				$url = isset( $menu_item['url'] ) ? $menu_item['url'] : '';
																				$target = isset( $menu_item['target'] ) ? $menu_item['target'] : '';
																				?>
																				<li class="menu-item">
																					<h4 class="menu-title">
																					<span class="input-title">
																						<input type="checkbox" class="ova_check_menu_item" value="1">
																						<span class="title">
																							<?php echo esc_html( $label ); ?>
																						</span>
																					</span>
																					<span class="icon"><i class="dashicons dashicons-arrow-down"></i></span>
																					</h4>
																					<div class="content">
																						<ul class="list-menu-item-field">
																							<li class="menu-item-field">
																								<label for=""><?php esc_html_e( 'Navigation Label', 'ova-megamenu' ); ?></label>
																								<input
																								type="text"
																								class="menu_item_title large-text"
																								name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_item][<?php echo $k_item; ?>][label]"
																								placeholder=""
																								value="<?php echo esc_attr( $label ); ?>" />
																							</li>
																							<li class="menu-item-field">
																								<label for=""><?php esc_html_e( 'Link', 'ova-megamenu' ); ?></label>
																								<input
																								type="text"
																								class="large-text"
																								name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_item][<?php echo $k_item; ?>][url]"
																								placeholder=""
																								value="<?php echo esc_attr( $url ); ?>" />
																							</li>
																							<li class="menu-item-field">
																								<label for=""><?php esc_html_e( 'Title Attribute', 'ova-megamenu' ); ?></label>
																								<input
																								type="text"
																								class="menu_item_title_attr large-text"
																								name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_item][<?php echo $k_item; ?>][title]"
																								placeholder=""
																								value="<?php echo esc_attr( $title ); ?>" />
																							</li>
																							<li class="menu-item-field">
																								<label for=""><?php esc_html_e( 'CSS Classes', 'ova-megamenu' ); ?></label>
																								<input
																								type="text"
																								class="menu_item_classes large-text"
																								name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_item][<?php echo $k_item; ?>][class]"
																								placeholder=""
																								value="<?php echo esc_attr( $class ); ?>" />
																							</li>
																							<li class="menu-item-field">
																								<label for="">
																								<input
																								type="checkbox"
																								name="ova_megamenu_<?php echo $tab_name; ?>[<?php echo $k; ?>][<?php echo $j; ?>][widget][<?php echo $key; ?>][menu_item][<?php echo $k_item; ?>][target]"
																								<?php checked( $target, '1' ); ?>
																								value="1" />
																								<?php esc_html_e( 'Open link in a new tab', 'ova-megamenu' ); ?>
																								</label>
																							</li>
																							<li class="menu-item-field footer">
																								<a class="ova_megamenu_remove_menu_item" href="#"><?php esc_html_e( 'Remove', 'ova-megamenu' ); ?></a>
																								<span class="separator">|</span>
																								<a class="ova_megamenu_cancel_menu_item" href="#"><?php esc_html_e( 'Cancel', 'ova-megamenu' ); ?></a>
																							</li>
																						</ul>
																					</div>
																				</li>
																			<?php endforeach; ?>
																		<?php endif; ?>
																	</ul>
																	<div class="ova_footer">
																		
																		<button type="button" class="
																		<?php echo esc_attr( $active_bulk_select ); ?> ova_megamenu_menu_item_bulk_select button button-small button-secondary">
																			<input type="checkbox" class="menu_item_bulk_select" value="1">
																			<span><?php esc_html_e( 'Bulk Select', 'ova-megamenu' ); ?></span>
																		</button>
																	
																		<a href="#" class="ova_menu_item_remove"><?php esc_html_e( 'Remove Selected Items', 'ova-megamenu' ); ?></a>
																		
																	</div>
																	

																	<button type="button"
																	data-btn="<?php esc_attr_e( 'Close', 'ova-megamenu' ); ?>"
																	data-title="<?php esc_attr_e( 'Add menu item', 'ova-megamenu' ); ?>" class="ova_megamenu_add_menu_item button button-secondary"><?php esc_html_e( 'Add menu item', 'ova-megamenu' ); ?></button>

																</td>
															</tr>


														</tbody>
													</table>

												</li>
											<?php endforeach; ?>
										<?php endif; ?>
									</ul>

										<button type="button" class="ova_megamenu_add_widget button button-secondary"><?php esc_html_e( 'Add widget', 'ova-megamenu' ); ?></button>

								</li>
							<?php endforeach; ?>
						<?php endif; ?>
					</ul>

					<button type="button" class="ova_megamenu_remove_row button button-small button-primary">&#x2715;</button>
						
					<button type="button" class="ova_megamenu_add_column button button-secondary"><?php esc_html_e( 'Add column', 'ova-megamenu' ); ?></button>
				</li>
			<?php endforeach;
		}
	}
}